<?php

namespace Controller;

use Zeedhi\Framework\DTO\Response\Message;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DTO\Request;

use Zeedhi\Framework\DataSource\DataSet;

class Menu extends \Zeedhi\Framework\Controller\Simple {
    
    const ERROR_MESSAGE = 'E';
    // -- Homologação
    //const URL_SERVIDOR_IMAGEM = 'https://midia.teknisa.com/files/';
    // -- Produção
    // const URL_SERVIDOR_IMAGEM = 'http://midia.teknisa.com/files/';

    const RESIZE_IMAGE_SMALL_WIDTH = 130;
    const RESIZE_IMAGE_SMALL_HEIGHT = 130;
    const RESIZE_IMAGE_LARGE_WIDTH = 550;
    const RESIZE_IMAGE_LARGE_HEIGHT = 550;

    const GENERAL_MESSAGE   = 'A';

    const TIPO_GRUPO = "2";
    const TIPO_SUB_GRUPO = "I";
    const OBS_TIPO_ADICIONAR = "A";

    // -- Endereço de homologação da api de integração com a plataforma CRM Bonus
    const CRM_BONUS_URL = "https://dev-integracoes.teknisa.com/crm-bonus/backend/service/index.php";
    
    // -- Endereço de produção da api de integração com a plataforma CRM Bonus
    // const CRM_BONUS_URL = "https://dev-integracoes.teknisa.com/crm-bonus/backend/service/index.php";
    
    protected $util;
    protected $menuService;
    protected $orderService;
    
    public function __construct(\Util\Util $util, 
                                \Service\Menu $menuService, 
                                \Service\Order $orderService,
                                \Zeedhi\Framework\DependencyInjection\InstanceManager $instanceManager) {

        $this->util              = $util;
        $this->menuService       = $menuService;
        $this->orderService      = $orderService;
        $this->instanceManager   = $instanceManager;
        $this->orderTimeApiUrl   = $instanceManager->getParameter('ORDER_TIME_API_URL');
        $this->orderTimeApiToken = $instanceManager->getParameter('ORDER_TIME_API_TOKEN');
        $this->fileserverUrl     = $util->getFileServerURL();
    }

    /*  
        Retorna o cardápio de um caixa (caixa/loja/filial) no momento em que a rotina é consultada. 
    */
    public function getMenuByPos(Request $request, Response $response) {
        
        ini_set('zlib.output_compression_level', 1);
        ob_start('ob_gzhandler');
        
        $inputJSON = file_get_contents('php://input');
        $input     = json_decode($inputJSON, true);

        try {
            if (!isset($input)) {
                throw new \Exception('Não foram recebidos parâmetros');
            }
            
            $data           = array(); // dados a serem retornados
            $params         = $input;
            $subsidiaryCode = $params['subsidiaryCode'];
            $storeCode      = $params['storeCode'];
            $posCode        = $params['posCode'];
            $consumerType   = $params['consumerType'];
            $maskedHour     = $params['hour'];
            $hour           = str_replace(':','',$maskedHour);
            
            // verifica a existência da filial
            if (!$this->util->subsidiaryExists($subsidiaryCode)) {
                throw new \Exception('Filial inexistente.');
            }
            
            // verifica a existência da loja relacionada à filial
            if (!$this->util->storeExists($subsidiaryCode, $storeCode)) {
                throw new \Exception('Loja inexistente.');
            }
            
            // verifica a existência do caixa relacionado à loja
            if (!$this->util->posExists($subsidiaryCode, $storeCode, $posCode)) {
                throw new \Exception('Caixa inexistente.');
            }
            
            // busca pela tabela de preços da loja
            $return = $this->menuService->getActivePrincingTable($subsidiaryCode, $storeCode);
            
            // verifica a existência de tabela de preços cadastrada e ativa
            if (!$return['error']) {
                // dados necessários para pesquisar o preço de cada item do cardápio
                $pricingTableCode = $return['CDTABEPREC'];
                $pricingTableDate = $return['DTINIVGPREC'];
                // array com o preço de todos os produtos da loja, independente de grupos ou subgrupos
                $productPrices    = array();
                
                // busca o preço de cada produto da filial segundo o tipo do consumidor
                $result = $this->menuService->getPrices($subsidiaryCode, $pricingTableCode, $pricingTableDate, $consumerType, $hour, false);

                if (!$result['error']) {
                    $productPrices = $result['data'];
                } else {
                    $message = $result['message'];
                    $response->addMessage(new Message($message, self::ERROR_MESSAGE));
                }
                
                // busca os grupos de produtos do cardápio do caixa
                $return = $this->menuService->getGroups($subsidiaryCode, $posCode);
                
                if (!$return['error']) {
                    $groups = $return['data'];
                    
                    foreach ($groups as $group) {
                        $groupImage = $group['DSENDEIMG'] != null ? $this->fileserverUrl . $group['DSENDEIMG'] : null;
                        $groupData = array(
                            'groupName'         => $group['NMGRUPO'],
                            'groupImage'        => $groupImage,
                            'groupCode'         => $group['NRBUTTON'],
                            'groupDescription'  => $group['DSITMENUVENDA'],
                            'products'          => array(),
                            'groupIcon'         => $group['DSENDEIMGAUX'],
                            'subGroups'         => array()
                        );
                        
                        $groupParams = array(
                            'NRPGCONFTELA' => $group['NRPGCONFTELA'],
                            'NRBUTTON'     => $group['NRBUTTON']
                        );
                        
                        // busca os produtos de cada grupo
                        $return = $this->menuService->getGroupProducts($subsidiaryCode, $posCode, $groupParams, $storeCode);
                        if (!$return['error']) {
                            $products = $return['data'];
                            
                            foreach ($products as $product) {
                                if(isset($productPrices[$product['CDPRODUTO']])){
                                    $productImage = $product['DSENDEIMG'] != null ? $this->fileserverUrl . $product['DSENDEIMG'] : null;
                                    $groupProductData = array(
                                        'barCode' => $product['CDBARPRODUTO'],
                                        'productCode' => $product['CDPRODUTO'],
                                        'productName' => $product['NMPRODUTO'],
                                        'productDescription' => $product['DSPRODVENDA'] == null ? "" : $product['DSPRODVENDA'],
                                        'hasComboProducts' => $product['IDTIPOCOMPPROD'] == '3',
                                        'productImage' => $productImage,
                                        'price'       => 0.0,
                                        'cdarvprod'   => $product['CDARVPROD']
                                    );
                                    $hasObservations = $this->menuService->checkProductObservations($product['CDPRODUTO'], $subsidiaryCode, $storeCode);
                                    
                                    $groupProductData['hasObservations'] = $hasObservations;
                                    if (isset($product['CDPRODUTO'])) {
                                        $price = $productPrices[$product['CDPRODUTO']]['VRPRECITEM'];
                                        if (!empty($price)) {
                                            // arredonda o valor com precisão de duas casas decimais
                                            $price = round(floatval($price), 2);
                                            $groupProductData['price'] = $price;
                                            
                                            // só adiciona produtos sem problemas com preços
                                            array_push($groupData['products'], $groupProductData);
                                        } else {
                                            // $groupProductData['price'] = $this->util->float('0.00');
                                            
                                            // $message = 'O preço do produto '.$product['CDPRODUTO'].' está zerado';
                                            // $response->addMessage(new Message($message, self::ERROR_MESSAGE));
                                        }
                                    } else {
                                        // $message = 'Não há preço cadastrado para o produto '.$product['CDPRODUTO'];
                                        // $response->addMessage(new Message($message, self::ERROR_MESSAGE));
                                    }
                                }
                            }

                        } else {
                            // Erro ao buscar produtos do grupo
                            $message = $return['message'];
                            $response->addMessage(new Message($message, self::ERROR_MESSAGE));
                        }
                        // busca os subgrupos de cada grupo
                        $return = $this->menuService->getSubGroups($subsidiaryCode, $posCode, $groupParams);
                        
                        if (!$return['error']) {
                            $subGroups = $return['data'];
                            // --- temporário
                            $subGroupData = array();
                            // ---
                            foreach ($subGroups as $subGroup) {
                                $subGroupData = array(
                                    'name'     => $subGroup['NMGRUPO'],
                                    'products' => array()
                                );
                                
                                $subGroupParams = array(
                                    'NRPGCONFTELA' => $subGroup['NRPGCONFTELA'],
                                    'NRBUTTON'     => $subGroup['NRBUTTON'],
                                    'NRPGCONFTAUX' => $subGroup['NRPGCONFTAUX'],
                                    'NRBUTTONAUX'  => $subGroup['NRBUTTONAUX']
                                );
                                
                                // busca os produtos de cada subgrupo
                                $return = $this->menuService->getSubGroupProducts($subsidiaryCode, $posCode, $groupParams, $subGroupParams, $storeCode);
                                
                                if (!$return['error']) {
                                    $products = $return['data'];
                                    
                                    foreach ($products as $product) {
                                        $productImage = $product['DSENDEIMG'] != null ? $this->fileserverUrl . $product['DSENDEIMG'] : null;
                                        $subgroupProductData = array(
                                            'barCode' => $product['CDBARPRODUTO'],
                                            'productCode' => $product['CDPRODUTO'],
                                            'productName' => $product['NMPRODUTO'],
                                            'productDescription' => $product['DSPRODVENDA'] == null ? "" : $product['DSPRODVENDA'],
                                            'hasComboProducts' => $product['IDTIPOCOMPPROD'] == '3',
                                            'productImage' => $productImage, 
                                            'price' => 0.0
                                        );
                                        
                                        $hasObservations = $this->menuService->checkProductObservations($product['CDPRODUTO'], $subsidiaryCode, $storeCode);
                                        $subgroupProductData['hasObservations'] = $hasObservations;
                                        
                                        $price = $productPrices[$product['CDPRODUTO']]['VRPRECITEM'];
                                        if (!empty($price)) {
                                            $subgroupProductData['price'] = $this->util->float($price);
                                        } else {
                                            $subgroupProductData['price'] = $this->util->float('0.00');
                                        }
                                        array_push($subGroupData['products'], $subgroupProductData);
                                    }
                                } else {
                                    // Erro ao buscar produtos do subgrupo
                                    $message = $return['message'];
                                    $response->addMessage(new Message($message, self::ERROR_MESSAGE));
                                }
                            }
                            
                            array_push($groupData['subGroups'], $subGroupData);
                        } else {
                            // Erro ao buscar subgrupos do grupo
                            $message = $return['message'];
                            $response->addMessage(new Message($message, self::ERROR_MESSAGE));
                        }
                        
                        array_push($data, $groupData);
                    } 
                    
                    $response->addDataSet(new DataSet('', $data));
                } else {
                    // Erro ao buscar grupos de produtos
                    $message = $return['message'];
                    $response->addMessage(new Message($message, self::ERROR_MESSAGE));
                }
            } else {
                $message = $return['message'];
                $response->addMessage(new Message($message, self::ERROR_MESSAGE));
            }
        } catch (\Exception $e) {
            $message = 'Erro ao buscar cardápio do caixa. '.$e->getMessage();
            $response->addMessage(new Message($message, self::ERROR_MESSAGE));
        }
    }

    private function buildCachedImages($fileName) {
        $separator = DIRECTORY_SEPARATOR;
        $basePath  = preg_replace('/[\/\\\\]/', $separator, $this->instanceManager->getParameter('IMAGES_PATH'));

        $largeFile = (file_exists("{$basePath}{$separator}L-{$fileName}") ? "IMAGES/L-{$fileName}" : null);
        $smallFile = (file_exists("{$basePath}{$separator}S-{$fileName}") ? "IMAGES/S-{$fileName}" : null);

        return [
            'large' => $largeFile,
            'small' => $smallFile
        ];        
    }

    /*  
        Retorna o cardápio de um caixa (caixa/loja/filial) no momento em que a rotina é consultada. 
    */
    public function getMenuByPosEx(Request $request, Response $response) {
        ini_set('zlib.output_compression_level', 1);
        ob_start('ob_gzhandler');

        $inputJSON = file_get_contents('php://input');
        $input = json_decode($inputJSON, true);

        try {
            if (!isset($input)) {
                throw new \Exception('Não foram recebidos parâmetros');
            }
            
            $data   = array(); // dados a serem retornados

            $params = $input;
            $subsidiaryCode = $params['subsidiaryCode'];
            $storeCode      = $params['storeCode'];
            $posCode        = $params['posCode'];
            $consumerType   = $params['consumerType'];
            $maskedHour     = $params['hour'];
            $hour           = str_replace(':','',$maskedHour);
            
            // verifica a existência da filial
            if (!$this->util->subsidiaryExists($subsidiaryCode)) {
                throw new \Exception('Filial inexistente.');
            }
            
            // verifica a existência da loja relacionada à filial
            if (!$this->util->storeExists($subsidiaryCode, $storeCode)) {
                throw new \Exception('Loja inexistente.');
            }
            
            // verifica a existência do caixa relacionado à loja
            if (!$this->util->posExists($subsidiaryCode, $storeCode, $posCode)) {
                throw new \Exception('Caixa inexistente.');
            }
            
            // busca pela tabela de preços da loja
            $return = $this->menuService->getActivePrincingTable($subsidiaryCode, $storeCode);
            
            // verifica a existência de tabela de preços cadastrada e ativa
            if (!$return['error']) {
                // dados necessários para pesquisar o preço de cada item do cardápio
                $pricingTableCode = $return['CDTABEPREC'];
                $pricingTableDate = $return['DTINIVGPREC'];
                // array com o preço de todos os produtos da loja, independente de grupos ou subgrupos
                $productPrices    = array();

                // busca o preço de cada produto da filial segundo o tipo do consumidor
                $result = $this->menuService->getPrices($subsidiaryCode, $pricingTableCode, $pricingTableDate, $consumerType, $hour, false);

                if (!$result['error']) {
                    $productPrices = $result['data'];
                } else {
                    $message = $result['message'];
                    $response->addMessage(new Message($message, self::ERROR_MESSAGE));
                }

                // busca os grupos de produtos do cardápio do caixa
                $return = $this->menuService->getGroups($subsidiaryCode, $posCode);
                
                if (!$return['error']) {
                    $groups = $return['data'];
                    
                    foreach ($groups as $group) {
                        $groupImage = $group['DSENDEIMG'] != null ? $this->fileserverUrl . $group['DSENDEIMG'] : null;
                        $groupData = array(
                            'groupName'         => $group['NMGRUPO'],
                            'groupImage'        => $groupImage,
                            'groupImages'       => $this->buildCachedImages(basename($groupImage)),
                            'groupCode'         => $group['NRBUTTON'],
                            'groupDescription'  => $group['DSITMENUVENDA'],
                            'products'          => array(),
                            'groupIcon'         => $group['DSENDEIMGAUX'],
                            'subGroups'         => array()
                        );
                        
                        $groupParams = array(
                            'NRPGCONFTELA' => $group['NRPGCONFTELA'],
                            'NRBUTTON'     => $group['NRBUTTON']
                        );
                        
                        // busca os produtos de cada grupo
                        $return = $this->menuService->getGroupProducts($subsidiaryCode, $posCode, $groupParams, $storeCode);
                        if (!$return['error']) {
                            $products = $return['data'];
                            
                            foreach ($products as $product) {

                                if(isset($productPrices[$product['CDPRODUTO']])){
                                    $productImage = $product['DSENDEIMG'] != null ? $this->fileserverUrl . $product['DSENDEIMG'] : null;
                                    $groupProductData = array(
                                        'barCode' => $product['CDBARPRODUTO'],
                                        'productCode' => $product['CDPRODUTO'],
                                        'productName' => $product['NMPRODUTO'],
                                        'productDescription' => $product['DSPRODVENDA'] == null ? "" : $product['DSPRODVENDA'],
                                        // 'hasComboProducts' => $product['IDTIPOCOMPPROD'] == '3',
                                        'productImage'     => $productImage,
                                        'productImages'    => $this->buildCachedImages(basename($productImage)),
                                        'productPrint'     => $product['IDIMPPRODUTO'] == "1" ? "S" : "N",
                                        'price'            => 0.0
                                    );

                                    $subProducts = [];
                                    if($product['IDTIPOCOMPPROD'] == '3'){
                                        $subProducts = $this->getProductSubProducts($subsidiaryCode, $storeCode, $groupParams['NRPGCONFTELA'], $product['CDPRODUTO'], $productPrices, true);
                                    } 

                                    $groupProductData['subProducts'] = $subProducts;
                                    // $hasObservations = $this->menuService->checkProductObservations($product['CDPRODUTO'], $subsidiaryCode, $storeCode);
                                    // $groupProductData['hasObservations'] = $hasObservations;
                                    
                                    $groupProductData['observationsChoice'] = $this->getProductObservationsToMenu($subsidiaryCode, $storeCode, $product['CDPRODUTO'], $productPrices);

                                    if (isset($product['CDPRODUTO'])) {
                                        $price = $productPrices[$product['CDPRODUTO']]['VRPRECITEM'];
                                        if (!empty($price)) {
                                            // arredonda o valor com precisão de duas casas decimais
                                            $price = round(floatval($price), 2);
                                            $groupProductData['price'] = $price;
                                            
                                            // só adiciona produtos sem problemas com preços
                                            array_push($groupData['products'], $groupProductData);
                                        } else {
                                            // $groupProductData['price'] = $this->util->float('0.00');
                                            
                                            // $message = 'O preço do produto '.$product['CDPRODUTO'].' está zerado';
                                            // $response->addMessage(new Message($message, self::ERROR_MESSAGE));
                                        }
                                    } else {
                                        // $message = 'Não há preço cadastrado para o produto '.$product['CDPRODUTO'];
                                        // $response->addMessage(new Message($message, self::ERROR_MESSAGE));
                                    }
                                }
                                
                            }

                        } else {
                            // Erro ao buscar produtos do grupo
                            $message = $return['message'];
                            $response->addMessage(new Message($message, self::ERROR_MESSAGE));
                        }
                        
                        // busca os subgrupos de cada grupo
                        $return = $this->menuService->getSubGroups($subsidiaryCode, $posCode, $groupParams);
                        
                        if (!$return['error']) {
                            $subGroups = $return['data'];
                            // --- temporário
                            $subGroupData = array();
                            // ---
                            foreach ($subGroups as $subGroup) {
                                $subGroupImage = $subGroup['DSENDEIMG'] != null ? $this->fileserverUrl . $subGroup['DSENDEIMG'] : null;
                                $subGroupData = array(
                                    'name'     => $subGroup['NMGRUPO'],
                                    'subGroupImage'        => $subGroupImage,
                                    'subGroupImages'       => $this->buildCachedImages(basename($subGroupImage)),
                                    'subGroupIcon'         => $subGroup['DSENDEIMGAUX'],
                                    'products' => array(),
                                );
                                
                                $subGroupParams = array(
                                    'NRPGCONFTELA' => $subGroup['NRPGCONFTELA'],
                                    'NRBUTTON'     => $subGroup['NRBUTTON'],
                                    'NRPGCONFTAUX' => $subGroup['NRPGCONFTAUX'],
                                    'NRBUTTONAUX' => $subGroup['NRBUTTONAUX']
                                );
                                
                                // busca os produtos de cada subgrupo
                                $return = $this->menuService->getSubGroupProducts($subsidiaryCode, $posCode, $groupParams, $subGroupParams, $storeCode);
                                
                                if (!$return['error']) {
                                    $products = $return['data'];
                                    
                                    foreach ($products as $product) {
                                        $productImage = $product['DSENDEIMG'] != null ? $this->fileserverUrl . $product['DSENDEIMG'] : null;
                                        $subgroupProductData = array(
                                            'barCode' => $product['CDBARPRODUTO'],
                                            'productCode' => $product['CDPRODUTO'],
                                            'productName' => $product['NMPRODUTO'],
                                            'productDescription' => $product['DSPRODVENDA'] == null ? "" : $product['DSPRODVENDA'],
                                            
                                            // 'hasComboProducts' => $product['IDTIPOCOMPPROD'] == '3',
                                            'productImage'     => $productImage,
                                            'productImages'    => $this->buildCachedImages(basename($productImage)),
                                            'productPrint'     => $product['IDIMPPRODUTO'] == "1" ? "S" : "N",
                                            'price' => 0.0
                                        );

                                        $subProducts = [];
                                        if($product['IDTIPOCOMPPROD'] == '3'){
                                            $subProducts = $this->getProductSubProducts($subsidiaryCode, $storeCode, $groupParams['NRPGCONFTELA'], $product['CDPRODUTO'], $productPrices, true);
                                        } 

                                        $subgroupProductData['subProducts'] = $subProducts;
                                        // $hasObservations = $this->menuService->checkProductObservations($product['CDPRODUTO'], $subsidiaryCode, $storeCode);
                                        // $subgroupProductData['hasObservations'] = $hasObservations;
                                        $subgroupProductData['observationsChoice'] = $this->getProductObservationsToMenu($subsidiaryCode, $storeCode, $product['CDPRODUTO'], $productPrices);
                                                                                
                                        $price = (!empty($productPrices[$product['CDPRODUTO']])) ?  $productPrices[$product['CDPRODUTO']]['VRPRECITEM'] : 0;
                                        if (!empty($price)) {
                                            $subgroupProductData['price'] = $this->util->float($price);
                                        } else {
                                            $subgroupProductData['price'] = $this->util->float('0.00');
                                        }
                                        array_push($subGroupData['products'], $subgroupProductData);
                                    }
                                } else {
                                    // Erro ao buscar produtos do subgrupo
                                    $message = $return['message'];
                                    $response->addMessage(new Message($message, self::ERROR_MESSAGE));
                                }
                                
                                array_push($groupData['subGroups'], $subGroupData);
                            }
                            
                            if(empty($groupData['subGroups'][0])) $groupData['subGroups'] = [];
                            
                        } else {
                            // Erro ao buscar subgrupos do grupo
                            $message = $return['message'];
                            $response->addMessage(new Message($message, self::ERROR_MESSAGE));
                        }
                        
                        array_push($data, $groupData);
                    } 
                    
                    $response->addDataSet(new DataSet('', $data));
                } else {
                    // Erro ao buscar grupos de produtos
                    $message = $return['message'];
                    $response->addMessage(new Message($message, self::ERROR_MESSAGE));
                }
            } else {
                $message = $return['message'];
                $response->addMessage(new Message($message, self::ERROR_MESSAGE));
            }
        } catch (\Exception $e) {
            $message = 'Erro ao buscar cardápio do caixa. '.$e->getMessage();
            $response->addMessage(new Message($message, self::ERROR_MESSAGE));
        }
    }

    private function getProductSubProducts($subsidiaryCode, $storeCode, $confTela, $productCode, $prices, $recursive){
        // if($recursive == false) return [];
        $returnSubProductsGroups = $this->menuService->getSubProductsGroups($productCode, $subsidiaryCode);

        $subProductsGroupsList = array();
        if($returnSubProductsGroups['error'])
            return [];

        $subProductsGroups = $returnSubProductsGroups['data'];
        foreach($subProductsGroups as $group){

            $groupData = array(
                'subProductsGroupCode' => $group['CDGRUPROMOC'],
                'subProductsGroupName' => $group['NMGRUPROMOC'],
                'subProductsGroupMin'  => intval($group['QUANTMIN']),
                'subProductsGroupMax'  => intval($group['QUANTIDADE']),
                'subProductsGroupMutex'   => $group['CDGRUPMUTEX'],
                'subProductsGroupPrint'=> $group['IDIMPGRPROMO'], 
                'subProductsGroupItems' => []
            );
            
            $returnSubProducts = $this->menuService->getSubProducts($subsidiaryCode, $confTela, $productCode, $group['NRSEQGRPROMO'], $group['CDGRUPROMOC']);
            $subProductsList = array();
            
            if(!$returnSubProducts['error']){
                $subProducts = $returnSubProducts['data'];
                foreach($subProducts as $subProduct) {
                    $subProductData = array(
                        'subItemBarCode' => $subProduct['CDARVPROD'],
                        'subItemCode' => $subProduct['CDPRODUTO'],
                        'subItemName' => $subProduct['NMPRODUTO'],
                        'subItemPreSelected' => $subProduct['OBRGRUPR'] == "S" ? "S" : "N",
                        'subItemPrice' => (isset($prices[$subProduct['CDPRODUTO']]['VRPRECITEM'])) ? $this->util->float($prices[$subProduct['CDPRODUTO']]['VRPRECITEM']) : 0,
                        'subItemObservationsChoice' => $this->getProductObservationsToMenu($subsidiaryCode, $storeCode, $subProduct['CDPRODUTO'], $prices),
                        'subItemProducts' => []
                    );
                    if(!empty($subProduct['IDAPLICADESCONTO']) && $subProduct['IDAPLICADESCONTO'] == "T" && !empty($subProduct['VRDESCONTO']) && $subProductData['subItemPrice'] != 0){
                        if($subProduct['IDPERCVALOR'] == 'V'){
                            if($subProduct['IDDESCACR'] == 'D') $subProductData['subItemPrice'] -= $subProduct['VRDESCONTO'];  
                            else if($subProduct['IDDESCACR'] == 'A') $subProductData['subItemPrice'] += $subProduct['VRDESCONTO'];  
                        } else if($subProduct['IDPERCVALOR'] == 'P') {
                            if($subProduct['IDDESCACR'] == 'D') $subProductData['subItemPrice'] = $subProductData['subItemPrice'] - ($subProductData['subItemPrice'] * $subProduct['VRDESCONTO']);  
                            else if($subProduct['IDDESCACR'] == 'A') $subProductData['subItemPrice'] = $subProductData['subItemPrice'] + ($subProductData['subItemPrice'] * $subProduct['VRDESCONTO']);
                        }
                    } 
                    $subProductData['subItemPrice'] = $this->util->float($subProductData['subItemPrice']);
                    // contempla combo do combo ( 3 nivel de produtos )
                    if($subProduct['IDTIPOCOMPPROD'] == '3' && $recursive == true){
                        $subProductData['subItemProducts'] = $this->getProductSubProducts($subsidiaryCode, $storeCode, $confTela, $subProduct['CDPRODUTO'], $prices, false);  
                    }
                    array_push($subProductsList, $subProductData);
                    
                }
                $groupData['subProductsGroupItems'] = $subProductsList;
            }
            array_push($subProductsGroupsList, $groupData);
            
        }
        return $subProductsGroupsList;
    }

    private function getProductObservationsToMenu($subsidiaryCode, $storeCode, $productCode, $productPrices) {
        $returnGroups = $this->menuService->getObservationGroupsEx($subsidiaryCode, $storeCode);           
        if($returnGroups['error'])
            return [];

        $observationGroups = $returnGroups['data'];
        $groupList = array();
        foreach($observationGroups as $group){
            $groupCode = $group['CDGRPOCOR'];
            $groupData = array(
                'obsGroupCode' => $group['CDGRPOCOR'],
                'obsGroupName' => $group['DSGRPOCOR'],
                'minQuantity'  => $group['NRMINOCORR'],
                'maxQuantity'  => $group['NRMAXOCORR']
            );
            $returnObservations = $this->menuService->getObservationsByProduct($groupCode, $productCode, $subsidiaryCode, $storeCode);
            $observationList = array();

            if(!$returnObservations['error']){
                $observations = $returnObservations['data'];
                foreach($observations as $observation) {
                    $observationData = array(
                        'obsCode' => $observation['CDOCORR'],
                        'obsName' => $observation['DSOCORR'],
                        'obsType' => $this->formatObservationType($observation['IDCONTROLAOBS']),
                        'price'   => null
                    );     

                    if($observation['CDPRODUTO'] != null){
                        $prodObsCode = $observation['CDPRODUTO'];
                        if(isset($productPrices[$prodObsCode])){
                            $price = $productPrices[$prodObsCode]['VRPRECITEM'];
                            if (!empty($price)) {
                                $observationData['price'] = $this->util->float($price);
                            }   
                        }
                    }
                    array_push($observationList, $observationData);
                }

                // Ordenação de observações por ordem numérica e alfanumérica
                usort($observationList, function($a, $b) {
                    $a = $a['obsCode'];
                    $b = $b['obsCode'];
                    $reA = "/[^a-zA-Z]/g";
                    $reN = "/[^0-9]/g";
                    $aA = str_replace($reA, "", $a);
                    $bA = str_replace($reA, "", $b);
                    if ($aA === $bA) {
                        $aN = intval(str_replace($reN, "", $a));
                        $bN = intval(str_replace($reN, "", $b));
                        return ($aN === $bN) ? 0 : ($aN > $bN ? 1 : -1);
                    } else {
                        return ($aA > $bA) ? 1 : -1;
                    }
                });
                
                usort($observationList, function($a, $b) {
                    $a = $a['obsType'];
                    $b = $b['obsType'];
                    $reA = "/[^a-zA-Z]/g";
                    $reN = "/[^0-9]/g";
                    $aA = str_replace($reA, "", $a);
                    $bA = str_replace($reA, "", $b);
                    if ($aA === $bA) {
                        $aN = intval(str_replace($reN, "", $a));
                        $bN = intval(str_replace($reN, "", $b));
                        return ($aN === $bN) ? 0 : ($aN > $bN ? 1 : -1);
                    } else {
                        return ($aA > $bA) ? 1 : -1;
                    }
                });

                $observationListEx = [];
                foreach($observationList as $observation) {
                    array_push($observationListEx, [
                        'obsCode' => $observation['obsCode'],
                        'obsName' => $observation['obsName'],
                        'price'   => $observation['price']
                    ]);
                }

                $observationList = [];
                $observationList = $observationListEx;
                
            }

            if(!empty($observationList)) {
                $groupData['observations'] = $observationList;
                array_push($groupList, $groupData);
            }
        }

        return $groupList;
    }

    private function formatObservationType($idControlaObs) {
        $fType = 1;
        
        switch($idControlaObs) {
            case 'A': 
                $fType = 1;
                break;
            case 'R': 
                $fType = 2;
                break;
            case 'O': 
                $fType = 3;
                break;
        }

        return $fType;
    }

    //==== TAA TEKNISA
    public function getProductObservationsEx(Request $request, Response $response){
        $inputJSON = file_get_contents('php://input');
        $input = json_decode($inputJSON, true);

        try {
            if (!isset($input)) {
                throw new \Exception('Não foram recebidos parâmetros');
            }
            
            $data   = array(); // dados a serem retornados

            $params = $input;
            
            $subsidiaryCode = $params['subsidiaryCode'];
            $storeCode      = $params['storeCode'];
            $consumerType   = $params['consumerType'];
            $productCode    = $params['productCode'];
            $maskedHour     = $params['hour'];

            $hour           = str_replace(':','',$maskedHour);
            
            // verifica a existência da filial
            if (!$this->util->subsidiaryExists($subsidiaryCode)) {
                throw new \Exception('Filial inexistente.');
            }
            
            // verifica a existência da loja relacionada à filial
            if (!$this->util->storeExists($subsidiaryCode, $storeCode)) {
                throw new \Exception('Loja inexistente.');
            }

            // busca pela tabela de preços da loja
            $return = $this->menuService->getActivePrincingTable($subsidiaryCode, $storeCode);
            $productPrices = array();
            // verifica a existência de tabela de preços cadastrada e ativa
            if (!$return['error']) {
                // dados necessários para pesquisar o preço de cada item do cardápio
                $pricingTableCode = $return['CDTABEPREC'];
                $pricingTableDate = $return['DTINIVGPREC'];
             
                // busca o preço de cada produto da filial segundo o tipo do consumidor
                $return = $this->menuService->getPrices($subsidiaryCode, $pricingTableCode, $pricingTableDate, $consumerType, $hour, false);

                if(!$return['error']){
                    $productPrices = $return['data'];
                }
            }
                
            $returnGroups = $this->menuService->getObservationGroups($subsidiaryCode, $storeCode);           
            if(!$returnGroups['error']){
                $observationGroups = $returnGroups['data'];
                $groupList = array();
                foreach($observationGroups as $group){
                    $groupCode = $group['CDGRPOCOR'];
                    $groupData = array(
                        'obsGroupCode' => $group['CDGRPOCOR'],
                        'obsGroupName' => $group['DSGRPOCOR'],
                        'minQuantity'  => $group['NRMINOCORR'],
                        'maxQuantity'  => $group['NRMAXOCORR']
                    );
                    $returnObservations = $this->menuService->getObservationsByProduct($groupCode, $productCode, $subsidiaryCode, $storeCode);
                    $observationList = array();

                    if(!$returnObservations['error']){
                        $observations = $returnObservations['data'];
                        foreach($observations as $observation) {
                            $observationData = array(
                                'obsCode' => $observation['CDOCORR'],
                                'obsName' => $observation['DSOCORR'],
                                'obsType' => $this->formatObservationType($observation['IDCONTROLAOBS']),
                                'price'   => null
                            );     

                            if($observation['CDPRODUTO'] != null){
                                $prodObsCode = $observation['CDPRODUTO'];
                                if(isset($productPrices[$prodObsCode])){
                                    $price = $productPrices[$prodObsCode]['VRPRECITEM'];
                                    if (!empty($price)) {
                                        $observationData['price'] = $this->util->float($price);
                                    }   
                                }
                            }
                            array_push($observationList, $observationData);
                        }

                        // Ordenação de observações por ordem numérica e alfanumérica
                        usort($observationList, function($a, $b) {
                            $a = $a['obsCode'];
                            $b = $b['obsCode'];
                            $reA = "/[^a-zA-Z]/g";
                            $reN = "/[^0-9]/g";
                            $aA = str_replace($reA, "", $a);
                            $bA = str_replace($reA, "", $b);
                            if ($aA === $bA) {
                                $aN = intval(str_replace($reN, "", $a));
                                $bN = intval(str_replace($reN, "", $b));
                                return ($aN === $bN) ? 0 : ($aN > $bN ? 1 : -1);
                            } else {
                                return ($aA > $bA) ? 1 : -1;
                            }
                        });
                        
                        usort($observationList, function($a, $b) {
                            $a = $a['obsType'];
                            $b = $b['obsType'];
                            $reA = "/[^a-zA-Z]/g";
                            $reN = "/[^0-9]/g";
                            $aA = str_replace($reA, "", $a);
                            $bA = str_replace($reA, "", $b);
                            if ($aA === $bA) {
                                $aN = intval(str_replace($reN, "", $a));
                                $bN = intval(str_replace($reN, "", $b));
                                return ($aN === $bN) ? 0 : ($aN > $bN ? 1 : -1);
                            } else {
                                return ($aA > $bA) ? 1 : -1;
                            }
                        });

                        $observationListEx = [];
                        foreach($observationList as $observation) {
                            array_push($observationListEx, [
                                'obsCode' => $observation['obsCode'],
                                'obsName' => $observation['obsName'],
                                'price'   => $observation['price']
                            ]);
                        }

                        $observationList = [];
                        $observationList = $observationListEx;
                        
                    }

                    if(!empty($observationList)) {
                        $groupData['observations'] = $observationList;
                        array_push($groupList, $groupData);
                    }
                }

                $response->addDataSet(new DataSet('', $groupList));
            }else{
                $message = $returnGroups['message'];
                $response->addMessage(new Message($message, self::ERROR_MESSAGE));
            }
        } catch (\Exception $e) {
            $message = 'Erro ao buscar observacoes do produto. '.$e->getMessage();
            $response->addMessage(new Message($message, self::ERROR_MESSAGE));
        }
    }
 
    public function getProductObservations(Request $request, Response $response){
        $inputJSON = file_get_contents('php://input');
        $input = json_decode($inputJSON, true);

        try {
            if (!isset($input)) {
                throw new \Exception('Não foram recebidos parâmetros');
            }
            
            $data   = array(); // dados a serem retornados

            $params = $input;
            
            $subsidiaryCode = $params['subsidiaryCode'];
            $storeCode      = $params['storeCode'];
            $consumerType   = $params['consumerType'];
            $productCode    = $params['productCode'];
            $maskedHour     = $params['hour'];

            $hour           = str_replace(':','',$maskedHour);
            
            // verifica a existência da filial
            if (!$this->util->subsidiaryExists($subsidiaryCode)) {
                throw new \Exception('Filial inexistente.');
            }
            
            // verifica a existência da loja relacionada à filial
            if (!$this->util->storeExists($subsidiaryCode, $storeCode)) {
                throw new \Exception('Loja inexistente.');
            }

            // busca pela tabela de preços da loja
            $return = $this->menuService->getActivePrincingTable($subsidiaryCode, $storeCode);
            $productPrices = array();
            // verifica a existência de tabela de preços cadastrada e ativa
            if (!$return['error']) {
                // dados necessários para pesquisar o preço de cada item do cardápio
                $pricingTableCode = $return['CDTABEPREC'];
                $pricingTableDate = $return['DTINIVGPREC'];
             
                // busca o preço de cada produto da filial segundo o tipo do consumidor
                $return = $this->menuService->getPrices($subsidiaryCode, $pricingTableCode, $pricingTableDate, $consumerType, $hour, false);

                if(!$return['error']){
                    $productPrices = $return['data'];
                }
            }
                
            $returnGroups = $this->menuService->getObservationGroups($subsidiaryCode, $storeCode);           
            if(!$returnGroups['error']){
                $observationGroups = $returnGroups['data'];
                $groupList = array();
                foreach($observationGroups as $group){
                    $groupCode = $group['CDGRPOCOR'];
                    $groupData = array(
                        'obsGroupCode' => $group['CDGRPOCOR'],
                        'obsGroupName' => $group['DSGRPOCOR'],
                        'minQuantity'  => $group['NRMINOCORR'],
                        'maxQuantity'  => $group['NRMAXOCORR']
                    );
                    $returnObservations = $this->menuService->getObservationsByProduct($groupCode, $productCode, $subsidiaryCode, $storeCode);
                    $observationList = array();
                    if(!$returnObservations['error']){
                        $observations = $returnObservations['data'];
                        foreach($observations as $observation){
                            $observationData = array(
                                'obsCode' => $observation['CDOCORR'],
                                'obsName' => $observation['DSOCORR'],
                                'price'   => null
                            );     

                            if($observation['CDPRODUTO'] != null){
                                $prodObsCode = $observation['CDPRODUTO'];
                                if(isset($productPrices[$prodObsCode])){
                                    $price = $productPrices[$prodObsCode]['VRPRECITEM'];
                                    if (!empty($price)) {
                                        $observationData['price'] = $this->util->float($price);
                                    }   
                                }
                            }
                            array_push($observationList, $observationData);
                        }

                        // Ordenação de observações por ordem numérica e alfanumérica
                        usort($observationList, function($a, $b) {
                            $a = $a['obsCode'];
                            $b = $b['obsCode'];
                            $reA = "/[^a-zA-Z]/g";
                            $reN = "/[^0-9]/g";
                            $aA = str_replace($reA, "", $a);
                            $bA = str_replace($reA, "", $b);
                            if ($aA === $bA) {
                                $aN = intval(str_replace($reN, "", $a));
                                $bN = intval(str_replace($reN, "", $b));
                                return ($aN === $bN) ? 0 : ($aN > $bN ? 1 : -1);
                            } else {
                                return ($aA > $bA) ? 1 : -1;
                            }
                        });
                        
                        // usort($observationList, function($a, $b) {
                        //     if($a['price'] < $b['price'])
                        //         return 1;
                        //     return -1;
                        // });
                    }
                    if(!empty($observationList)){
                        $groupData['observations'] = $observationList;
                        array_push($groupList, $groupData);
                    }
                }
                $response->addDataSet(new DataSet('', $groupList));
            }else{
                $message = $returnGroups['message'];
                $response->addMessage(new Message($message, self::ERROR_MESSAGE));
            }
        } catch (\Exception $e) {
            $message = 'Erro ao buscar observacoes do produto. '.$e->getMessage();
            $response->addMessage(new Message($message, self::ERROR_MESSAGE));
        }
    }

    public function getProductCombos(Request $request, Response $response){
        $inputJSON = file_get_contents('php://input');
        $input = json_decode($inputJSON, true);

        try {
            if (!isset($input)) {
                throw new \Exception('Não foram recebidos parâmetros');
            }
            
            $data   = array(); // dados a serem retornados

            $params = $input;
            $subsidiaryCode = $params['subsidiaryCode'];
            $storeCode      = $params['storeCode'];
            $posCode        = $params['posCode'];
            $consumerType   = $params['consumerType'];
            $productCode    = $params['productCode'];
            $maskedHour     = $params['hour'];
            $hour           = str_replace(':','',$maskedHour);
            
            // verifica a existência da filial
            if (!$this->util->subsidiaryExists($subsidiaryCode)) {
                throw new \Exception('Filial inexistente.');
            }
            
            // verifica a existência da loja relacionada à filial
            if (!$this->util->storeExists($subsidiaryCode, $storeCode)) {
                throw new \Exception('Loja inexistente.');
            }
            
            // verifica a existência do caixa relacionado à loja
            if (!$this->util->posExists($subsidiaryCode, $storeCode, $posCode)) {
                throw new \Exception('Caixa inexistente.');
            }

            // busca pela tabela de preços da loja
            $return = $this->menuService->getActivePrincingTable($subsidiaryCode, $storeCode);
            $productPrices = array();
            $productConsumerPrices = array();
            // verifica a existência de tabela de preços cadastrada e ativa
            if (!$return['error']) {
                // dados necessários para pesquisar o preço de cada item do cardápio
                $pricingTableCode = $return['CDTABEPREC'];
                $pricingTableDate = $return['DTINIVGPREC'];
             
                // busca o preço de cada produto da filial segundo o tipo do consumidor
                $return = $this->menuService->getPrices($subsidiaryCode, $pricingTableCode, $pricingTableDate, $consumerType, $hour, true);
                if(!$return['error']){
                    $productConsumerPrices = $return['data'];
                }
                // busca o preço de cada produto da filial segundo o tipo do consumidor
               /*  $consumerType = 'T';
                $return = $this->menuService->getPrices($subsidiaryCode, $pricingTableCode, $pricingTableDate, $consumerType, $hour);
                if(!$return['error']){
                    $productPrices = $return['data'];
                } */
            }

            $returnGroups = $this->menuService->getComboGroupsBySubsidiary($subsidiaryCode, $productCode);
            if(!$returnGroups['error']){
                
                $comboGroups = $returnGroups['data'];
                $useSubsidiaryGroups = true;
                if(empty($comboGroups)){
                    $useSubsidiaryGroups = false;
                    $returnGroups = $this->menuService->getComboGroupsByProduct($productCode);
                    
                    if(!$returnGroups['error']){
                        $comboGroups = $returnGroups['data'];
                    }else{
                        $message = $returnGroups['message'];
                        $response->addMessage(new Message($message, self::ERROR_MESSAGE));
                    }   
                }


                $groupList = array();

                foreach($comboGroups as $comboGroup){
                   
                    $comboGroupData = array(
                        'groupName'  => $comboGroup['NMGRUPROMOC'],
                        'groupImage' => null,
                        'quantity'   => $comboGroup['QTPRGRUPPROMOC'],
                        'products'   => array()
                    );

                    $comboGroupCode = $comboGroup['CDGRUPROMOC'];

                    if($useSubsidiaryGroups){
                        
                        $return = $this->menuService->getComboProductsBySubsidiary($subsidiaryCode, $storeCode, $comboGroupCode, $productCode);
                    }else{
                        $return = $this->menuService->getComboProductsByProduct($subsidiaryCode, $storeCode, $comboGroupCode, $productCode, $pricingTableCode);
                    }

                    $comboProductsList = array();
                    if(!$return['error']){
                        $comboProducts = $return['data']; 
                        foreach($comboProducts as $comboProduct){
                            $comboProductData = array(
                                'barCode'            => $comboProduct['CDBARPRODUTO'],
                                'productCode'        => $comboProduct['CDPRODUTO'],
                                'productName'        => $comboProduct['NMPRODUTO'],
                                'productDescription' => $comboProduct['DSPRODVENDA'],
                                'hasComboProducts'   => false,
                                'productImage'       => $comboProduct['DSENDEIMG'] != null ? $this->fileserverUrl . $comboProduct['DSENDEIMG'] : null,
                                'price'              => 0
                            );
                            $currentProductCode = $comboProduct['CDPRODUTO'];

                            $hasObservations = $this->menuService->checkProductObservations($currentProductCode, $subsidiaryCode, $storeCode);

                            $comboProductData['hasObservations'] = $hasObservations;

                            $usePercentDiscount = $comboProduct['IDPERVALORDES'] == 'P';
                            $discountValue      = $this->util->float($comboProduct['VRDESPRODPROMOC']);
                            $productPrice = 0;


                            if(isset($productConsumerPrices[$currentProductCode])){
                                $productPrice = $productConsumerPrices[$currentProductCode]['VRPRECITEM'];
                                if (!empty($productPrice) && $discountValue !== 0) {
                                    if($usePercentDiscount){
                                        $productPrice = $this->util->float($productPrice) - ($this->util->float($productPrice) * ($discountValue)/100);
                                    }else{
                                        $productPrice = $this->util->float($productPrice) - $discountValue;    
                                    }
                                }

                                $applyDiscount = $productConsumerPrices[$currentProductCode]['IDPERVALORPR'];
                                $disAcr = $productConsumerPrices[$currentProductCode]['IDDESCACREPR'];

                                if($disAcr !== null){
                                    if($disAcr == 'D'){
                                        if($applyDiscount == 'V'){
                                            $productPrice = $productPrice - $this->util->float($productConsumerPrices[$currentProductCode]['VRPRECODIA']); 
                                        }else{
                                            $productPrice =  $productPrice - ($productPrice * $this->util->float($productConsumerPrices[$currentProductCode]['VRPRECODIA'])/100); 
                                            
                                        }
                                    
                                    }else{
                                        if($applyDiscount == 'V'){
                                            $productPrice += $this->util->float($productConsumerPrices[$currentProductCode]['VRPRECODIA']); 
                                        }else{
                                            $productPrice =  $productPrice + ($productPrice * $this->util->float($productConsumerPrices[$currentProductCode]['VRPRECODIA'])/100); 
                                        }
                                    }
                                } 
                            }
                            $comboProductData['price'] = $this->util->float($productPrice);
                            if($comboProductData['price'] < 0.01) {
                                $comboProductData['price'] = 0.01;
                            }
                            array_push($comboProductsList, $comboProductData);
                        }

                        if(!empty($comboProductsList)){
                            $comboGroupData['products'] = $comboProductsList;
                            array_push($groupList, $comboGroupData);
                        }
                    }else{
                        $message = $return['message'];
                        $response->addMessage(new Message($message, self::ERROR_MESSAGE));
                    } 
                }

                $response->addDataSet(new DataSet('', $groupList));
            }else{
                $message = $returnGroups['message'];
                $response->addMessage(new Message($message, self::ERROR_MESSAGE));
            }
        } catch (\Exception $e) {
            $message = 'Erro ao buscar combos do produto. '.$e->getMessage();
            $response->addMessage(new Message($message, self::ERROR_MESSAGE));
        }
    }
    
    public function getWaitingTime(Request $request, Response $response){
        try{
            $inputJSON = file_get_contents('php://input');
            $input = json_decode($inputJSON, true);
            if (isset($input['subsidiaryCode'])) $subsidiaryCode = $input['subsidiaryCode'];
            else throw new \Exception('Filial não informada.');

            if (isset($input['storeCode'])) $stores = $input['storeCode'];
            else throw new \Exception('Lojas não informadas.');
            
            // $storeCode = array();

            // foreach($stores as $store){
            //     $returnWaitingTime = $this->menuService->getWaitingTime($subsidiaryCode, $store);
            //     if($returnWaitingTime['data'] == false || empty($returnWaitingTime['data']['VRTMPMEDPROD']) ) $returnWaitingTime = $this->menuService->getStoreWaitingTime($subsidiaryCode, $store);
            //     $time = $returnWaitingTime['data'];
            //     $storeCode[$store] = empty($time['VRTMPMEDPROD']) ? $time['VRTMPESPLJ'] : $time['VRTMPMEDPROD'];
            //     $storeCode[$store] = $this->util->convertToHoursMins($storeCode[$store], '%02d:%02d:00');
            // }

            $curl = curl_init();

            curl_setopt_array($curl, array(
                CURLOPT_URL => $this->orderTimeApiUrl.'/api/orderTime',
                CURLOPT_RETURNTRANSFER => true,
                CURLOPT_ENCODING => '',
                CURLOPT_MAXREDIRS => 10,
                CURLOPT_TIMEOUT => 0,
                CURLOPT_FOLLOWLOCATION => true,
                CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
                CURLOPT_CUSTOMREQUEST => 'POST',
                CURLOPT_POSTFIELDS =>$inputJSON,
                CURLOPT_HTTPHEADER => array(
                    'x-functions-key: '.$this->orderTimeApiToken,
                    'Content-Type: application/json'
                ),
                CURLOPT_SSL_VERIFYPEER => 0,
                CURLOPT_CONNECTTIMEOUT => 5,
                CURLOPT_TIMEOUT => 20,
            ));

            $responseCurl = curl_exec($curl);
            $err    = curl_errno($curl);
            $errmsg = curl_error($curl);

            curl_close($curl);

            if($err){
                if ($err === 28) {
                    throw new \Exception("<br>Nao foi possivel estabelecer conexao com a API orderTime.");
                } else {
                    throw new \Exception("<br>Ocorreu um erro na chamada da API orderTime:<br><br>" . $errmsg);
                }
            }

            $responseOrderTime = json_decode($responseCurl, true);

            if(!empty($responseOrderTime['error']) && $responseOrderTime['error']) throw new \Exception($responseOrderTime['message']);

            $storeCode = !empty($responseOrderTime['data']['storeCode']) ? $responseOrderTime['data']['storeCode'] : array();

            $response->addDataSet(new DataSet('', array('storeCode' => $storeCode)));            
        } catch (\Exception $e) {
            $this->util->saveLog($e->getMessage(),'Error','WAITING_TIME-');
            $message = 'Erro ao recuperar o tempo de espera. ' . $e->getMessage();
            $response->addMessage(new Message($message, self::ERROR_MESSAGE));
        }
    }

    /**
     * Request Images to Download
     * 
     * Receives data from request necessary to download images from
     * groups, subgroups and products, if they have an image registered.
     * If the request parameter saveLocal is true, it will create local
     * directories and save the files.
     * 
     * @param Request $request Object that receives data from the caller
     * @param Response $response Object that sends back data to the caller
     * 
     * @return array
     * @throws Exception
     * 
     * @access public
     * @author Rômulo Alves Lousada
     */
    public function downloadImages() {
        try {

            $this->util->saveLog('Início serviço cache de imagens','ImageCache/'.date("Y/m"),'IMGCACHE-');

            set_time_limit(900);

            $result = array();
            $saveLocal = true;

            $availablePosFromSubsidiary = $this->menuService->getAvailablePosFromSubsidiary();

            if(!empty($availablePosFromSubsidiary['error'])) {
                return 0;
            }

            foreach($availablePosFromSubsidiary['data'] as $posData) {
                $subsidiaryCode = $posData['CDFILIAL'];
                $storeCode = $posData["CDLOJA"];
                $posCode   = $posData["CDCAIXA"];

                $separator = DIRECTORY_SEPARATOR;
                $basePath = preg_replace('/[\/\\\\]/', $separator, $this->instanceManager->getParameter('IMAGES_PATH'));

                if(!file_exists($basePath)) {
                    $this->createDirectory($basePath);
                }

                $groups = $this->menuService->getGroups($subsidiaryCode, $posCode);
                
                if (array_key_exists('data', $groups) && sizeof($groups['data']) > 0) {
                    $indexedGroups = array();
    
                    foreach ($groups['data'] as $group) {
                        $groupDirectory = "{$basePath}{$separator}{$group['NRBUTTON']}";
                        
                        $imgUrl = $this->getImageUrl($group);
        
                        if ($imgUrl !== '') {
                            $indexedGroups[] = [
                                'url'       => $imgUrl,
                                'directory' => $basePath,
                                'code'      => $group['NRBUTTON'],
                                'fileName'  => basename($imgUrl)
                            ];
                        }
                    
                        $groupParams = array(
                            'NRPGCONFTELA'  => $group['NRPGCONFTELA'],
                            'NRBUTTON'      => $group['NRBUTTON']
                        );
        
                        $groupProducts = $this->menuService->getGroupProducts($subsidiaryCode, $posCode, $groupParams, $storeCode);
        
                        if (sizeof($groupProducts['data']) > 0) {
                            $groupProductDirectory = "{$groupDirectory}{$separator}products";
                            
                            $indexedProducts = array();
                            
                            foreach ($groupProducts['data'] as $product) {
                                $imgUrl = $this->getImageUrl($product);
                                
                                if ($imgUrl !== '') {
                                    $indexedProducts[] = [
                                        'url'       => $imgUrl,
                                        'directory' => $basePath,
                                        'code'      => $product['CDPRODUTO'],
                                        'fileName'  => basename($imgUrl)                                            
                                    ];
                                }
                            }
    
                            $result['product']['group'][$group['NRBUTTON']] = $this->getImage($indexedProducts, $saveLocal);
                        }
        
                        $subGroups = $this->menuService->getSubGroups($subsidiaryCode, $posCode, $groupParams);
        
                        if (sizeof($subGroups['data']) > 0) {
                            $subGroupDirectory = "{$groupDirectory}{$separator}subgroups";
                            $indexedSubGroups = array();
                            foreach ($subGroups['data'] as $subGroup) {

                                $imgUrl = $this->getImageUrl($subGroup);
                    
                                if ($imgUrl !== '') {
                                    $indexedSubGroups[] = [
                                        'url' => $imgUrl,
                                        'directory' => $basePath,
                                        'code' => $subGroup['NRBUTTON'],
                                        'fileName' => basename($imgUrl)                                            
                                    ];
                                }

                                $subGroupParams = array(
                                    'NRPGCONFTELA' => $subGroup['NRPGCONFTELA'],
                                    'NRBUTTON'     => $subGroup['NRBUTTON'],
                                    'NRPGCONFTAUX' => $subGroup['NRPGCONFTAUX'],
                                    'NRBUTTONAUX' => $subGroup['NRBUTTONAUX']
                                );
        
                                $subGroupProducts = $this->menuService->getSubGroupProducts($subsidiaryCode, $posCode, $groupParams, $subGroupParams, $storeCode);
        
                                if (sizeof($subGroupProducts['data']) > 0) {
                                    $subGroupProductDirectory = "{$subGroupDirectory}{$separator}products";
                                    
                                    $indexedSubGroupProducts = array();
                                    
                                    foreach ($subGroupProducts['data'] as $product) {
                                        $imgUrl = $this->getImageUrl($product);
                    
                                        if ($imgUrl !== '') {
                                            $indexedSubGroupProducts[] = [
                                                'url' => $imgUrl,
                                                'directory' => $basePath,
                                                'code' => $product['CDPRODUTO'],
                                                'fileName' => basename($imgUrl)                                            
                                            ];
                                        }
                                    }
    
                                    $result['product']['subgroup'][$subGroup['NRBUTTON']] = $this->getImage($indexedSubGroupProducts, $saveLocal);
                                }
                            }
                            $result['subgroup'] = $this->getImage($indexedSubGroups, $saveLocal);
                        }
                    }
                    $result['group'] = $this->getImage($indexedGroups, $saveLocal);
                }
            }

            $this->util->saveLog('Fim serviço cache de imagens','ImageCache/'.date("Y/m"),'IMGCACHE-');
        } catch (\Exception $e) {
            $message = 'Erro ao realizar o download das imagens. ' . $e->getMessage();
            $this->util->saveLog($message, 'ImageCache/'.date("Y/m"),'IMGCACHE-');
        }
    }

    /**
     * Request Images to Download
     * 
     * Receives data from request necessary to download images from
     * products, if they have an image registered.
     * If the request parameter saveLocal is true, it will create local
     * directories and save the files.
     * 
     * @param Request $request Object that receives data from the caller
     * @param Response $response Object that sends back data to the caller
     * 
     * @return array
     * @throws Exception
     * 
     * @access public
     * @author Heitor Dinalli Carvalho
     */

     public function downloadImagesEx() {
      try {

          $this->util->saveLog('Início serviço cache de imagens','ImageCache/'.date("Y/m"),'IMGCACHE-');

          set_time_limit(900);

          $result = array();
          $saveLocal = true;
    
          $conftelacaixa = $this->menuService->getConfTelaCaixaTaa();
          $availableProducts = $this->menuService->getAvailableProducts($conftelacaixa['data']);

          if($availableProducts['error']) {
              return 0;
          }

          $availableComboProducts = $this->menuService->getAvailableComboProducts();

          if($availableComboProducts['error']) {
              return 0;
          }

          $availableGroups = $this->menuService->getAvailableGroups();

          if($availableGroups['error']) {
            return 0;
          }

          $availableSubGroups = $this->menuService->getAvailableSubGroups();

          if($availableSubGroups['error']) {
            return 0;
          }

          $separator = DIRECTORY_SEPARATOR;
          $basePath = preg_replace('/[\/\\\\]/', $separator, $this->instanceManager->getParameter('IMAGES_PATH'));

          if(!file_exists($basePath)) {
              $this->createDirectory($basePath);
          }

          $indexedProducts = array();

          foreach($availableProducts['data'] as $product) {
              $imgUrl = $this->getImageUrl($product);

              if ($imgUrl !== '') {
                  $indexedProducts[] = [
                      'url'       => $imgUrl,
                      'directory' => $basePath,
                      'code'      => $product['CDPRODUTO'],
                      'fileName'  => basename($imgUrl)                                            
                  ];
              }
          }

          $indexedComboProducts = array();

          foreach($availableComboProducts['data'] as $comboProduct) {
                $imgUrl = $this->getImageUrl($comboProduct);
                if ($imgUrl !== '') {
                    $indexedComboProducts[] = [
                        'url'       => $imgUrl,
                        'directory' => $basePath,
                        'code'      => $comboProduct['CDPRODUTO'],
                        'fileName'  => basename($imgUrl)                                            
                    ];
              }
          }

          $indexedGroups = array();
          foreach ($availableGroups['data'] as $group) {            
            $imgUrl = $this->getImageUrl($group);

            if ($imgUrl !== '') {
                $indexedGroups[] = [
                    'url'       => $imgUrl,
                    'directory' => $basePath,
                    'code'      => $group['NRBUTTON'],
                    'fileName'  => basename($imgUrl)
                ];
            }
          }

          $indexedSubGroups = array();
          foreach ($availableSubGroups['data'] as $subGroup) {            
            $imgUrl = $this->getImageUrl($subGroup);

            if ($imgUrl !== '') {
                $indexedSubGroups[] = [
                    'url'       => $imgUrl,
                    'directory' => $basePath,
                    'code'      => $subGroup['NRBUTTON'],
                    'fileName'  => basename($imgUrl)
                ];
            }
          }

          $result = $this->getImage($indexedProducts, $saveLocal);
          $resultGroup = $this->getImage($indexedGroups, $saveLocal);
          $resultSubGroup = $this->getImage($indexedSubGroups, $saveLocal);
          $resultComboProducts = $this->getImage($indexedComboProducts, $saveLocal);
          

          $this->util->saveLog('Fim serviço cache de imagens','ImageCache/'.date("Y/m"),'IMGCACHE-');
      } catch (\Exception $e) {
          $message = 'Erro ao realizar o download das imagens. ' . $e->getMessage();
          $this->util->saveLog($message, 'ImageCache/'.date("Y/m"),'IMGCACHE-');
      }
    }   

    /**
     * Get images from received urls using cURL.
     * 
     * Receives data from request necessary to download and resize images from
     * groups, subgroups and products, if they have an image registered.
     * Will save local files if saveLocal is set to true. Otherwise, will
     * only return an array with the MIME base64.
     * 
     * @param Array $urls Contains an array with image data needed to 
     * download it. [ 'url', 'directory', 'code' ]
     * @param Boolean $saveLocal Will save local files inside the created
     * directories if set to true.
     * 
     * @return array
     * 
     * @access private
     * @author Rômulo Alves Lousada
     */
    private function getImage($urlsEx, $saveLocal) {
        $urls = [];
        foreach ($urlsEx as $data) {

            $largeFilename = "{$data['directory']}/L-{$data['fileName']}";
            $smallFilename = "{$data['directory']}/S-{$data['fileName']}";

            if (file_exists($largeFilename) && file_exists($smallFilename)) {
                //Atualiza o cache de em 48h, caso um novo upload seja realizado será baixado fora da regra abaixo
                if((time() - filemtime($smallFilename)) <= 172800) {
                    continue;
                }
            }          

            array_push($urls, $data);
        }

        $base64 = array();

        if (sizeof($urls) > 0) {
            $chs = array();
            $cmh = curl_multi_init();
            
            foreach ($urls as $i => $data) {
                $this->util->saveLog("Baixando {$data['fileName']}", 'ImageCache/'.date("Y/m"),'IMGCACHE-');
                $this->util->saveLog("link {$data['url']}", 'ImageCache/'.date("Y/m"),'IMGCACHE-');
                $chs[$i] = curl_init();

                curl_setopt($chs[$i], CURLOPT_URL, $data['url']);
                curl_setopt($chs[$i], CURLOPT_FOLLOWLOCATION, 1);
                curl_setopt($chs[$i], CURLOPT_RETURNTRANSFER, 1);
                curl_setopt($chs[$i], CURLOPT_SSL_VERIFYPEER, 0);
                curl_setopt($chs[$i], CURLOPT_TCP_KEEPALIVE, 1);
      
                curl_multi_add_handle($cmh, $chs[$i]);
            }
      
            $running = null;
      
            do {
                curl_multi_exec($cmh, $running);
            } while ($running > 0);

            foreach($urls as $i => $data) {

                $code = curl_getinfo($chs[$i], CURLINFO_HTTP_CODE);
                
                if ($code == 200) {
                    $mime = curl_getinfo($chs[$i], CURLINFO_CONTENT_TYPE);
                    $image = imagecreatefromstring(curl_multi_getcontent($chs[$i]));

                    if ($image !== false) {
                        $extension = $this->convertMimeToExt($mime);

                        if ($extension !== '') {
                            $smallSize = $this->getResizeDimensions($image, self::RESIZE_IMAGE_SMALL_WIDTH, self::RESIZE_IMAGE_SMALL_HEIGHT);
                            $largeSize = $this->getResizeDimensions($image, self::RESIZE_IMAGE_LARGE_WIDTH, self::RESIZE_IMAGE_LARGE_HEIGHT);
                            
                            if ($saveLocal) {
                                $smallPath = "{$data['directory']}/S-{$data['fileName']}";
                                $largePath = "{$data['directory']}/L-{$data['fileName']}";
        
                                $this->saveImage($image, $smallPath, $smallSize);
                                $this->saveImage($image, $largePath, $largeSize);
                            }
                        }
                    }
                }

                curl_multi_remove_handle($cmh, $chs[$i]);
                curl_close($chs[$i]);
            }

            curl_multi_close($cmh);
        }

        return $base64;
    }

    /**
     * Return Width and Height to Resize Image
     * 
     * Receives the image and the width/height to resize.
     * Calculates the actual image width/height ratio and adjusts the 
     * required width/heigth.
     * Then return an array with the actual image width/height and the
     * calculated resize width/height.
     * 
     * @param Resource $image Image created from cURL string with php gd2.
     * @param Int $resizeWidth Width to resize the image.
     * @param Int $resizeHeight Height to resize the image.
     * 
     * @return array
     * 
     * @access private
     * @author Rômulo Alves Lousada
     */
    private function getResizeDimensions($image, $resizeWidth, $resizeHeight)
    {
        $imageWidth = imagesx($image);
        $imageHeight = imagesy($image);
        $ratio = $imageWidth / $imageHeight;

        if ($ratio >= 1) {
            $resizeHeight = $resizeWidth / $ratio;
        } else {
            $resizeWidth = $resizeHeight * $ratio;
        }

        if ($resizeHeight < 130){
          $resizeHeight = $resizeHeight * $ratio;
          $resizeWidth = $resizeWidth * $ratio;
        }
        
        return [
          'imageWidth' => $imageWidth, 
          'imageHeight' => $imageHeight,
          'resizeWidth' => $resizeWidth, 
          'resizeHeight' => $resizeHeight
        ];
    }

    /**
     * Save Image Local.
     * 
     * Images gathered via cURL request are firstly removed from the directory, 
     * if it exists and then resized, respecting the ratio from the 
     * original image then saved in the informed directory. Resize values:
     * Small: 110 x 110
     * Large: 550 x 550
     * 
     * @param Resource $image Image created from cURL string with php gd2.
     * @param String $filepath Path where the image will be saved.
     * @param Array $sizes Width/Height size the image will be resized.
     * 
     * @return void
     * 
     * @access private
     * @author Rômulo Alves Lousada
     */
    private function saveImage($image, $filepath, $sizes) 
    {
        $this->removeFile($filepath);

        $newImage = imagecreatetruecolor($sizes['resizeWidth'], $sizes['resizeHeight']);

        imagefill($newImage, 0, 0, imagecolorallocate($newImage, 255, 255, 255));

        imagecopyresampled($newImage, $image, 0, 0, 0, 0, $sizes['resizeWidth'], $sizes['resizeHeight'], $sizes['imageWidth'], $sizes['imageHeight']);
        
        imagejpeg($newImage, $filepath);

        imagedestroy($newImage);
    }

    /**
     * Create a directory
     * 
     * Received a path and attempt to create it if it doesn't exist.
     * 
     * @param String $path Path to be created on the filesystem.
     * 
     * @return void
     * 
     * @access private
     * @author Rômulo Alves Lousada
     */
    private function createDirectory($path) 
    {
        if (!file_exists($path)) {
            mkdir($path, 0777, true);
        }
    }

    /**
     * Remove an Existing File
     * 
     * Attempt to remove a file if it exists.
     * 
     * @param String $file Path to the file to be removed.
     * 
     * @return void
     * 
     * @access private
     * @author Rômulo Alves Lousada
     */
    private function removeFile ($file) 
    {
        if (file_exists($file)) {
            unlink($file);
        }
    }

    /**
     * Get Image URL
     * 
     * If the image is not empty, concatenate with the initial url from
     * Teknisa File Server before returning it.
     * 
     * @param Array $image Contains the image address fields: ['DSENDEIMG', 'DSENDEIMGAUX']
     * 
     * @return String
     * 
     * @access private
     * @author Rômulo Alves Lousada
     */
    private function getImageUrl($image) 
    {
        
        if (array_key_exists('DSENDEIMG', $image) && $image['DSENDEIMG'] != '') {
            //return self::URL_SERVIDOR_IMAGEM . $image['DSENDEIMG'];
            return $this->fileserverUrl . $image['DSENDEIMG'];
        }

        if (array_key_exists('DSENDEIMGAUX', $image) && $image['DSENDEIMGAUX'] != '') {
            //return self::URL_SERVIDOR_IMAGEM . $image['DSENDEIMGAUX'];
            return $this->fileserverUrl . $image['DSENDEIMGAUX'];
        }

        return '';
    }

    /**
     * Convert Mimetype to Extension
     * 
     * Receives a mimetype gathered from the cURL request and attempt to
     * convert it to the respective file extension.
     * 
     * @param String $mime Mimetype from cURL request.
     * 
     * @return String
     * 
     * @access private
     * @author Rômulo Alves Lousada
     */
    private function convertMimeToExt ($mime) 
    {
        $mimeMap = [
            'image/gif' => 'gif',
            'image/jpeg' => 'jpeg',
            'image/pjpeg' => 'jpeg',
            'image/png' => 'png',
            'image/x-png' => 'png'
        ];

        return isset($mimeMap[$mime]) ? $mimeMap[$mime] : '';
    }

    public function V3_getProductsPrinterParams(Request $request, Response $response) {
        $inputJSON = file_get_contents('php://input');
        $input = json_decode($inputJSON, true);

        try {
            if (!isset($input)) {
                throw new \Exception('Não foram recebidos parâmetros');
            }
            
            $data   = array(); // dados a serem retornados

            $params = $input;
            $subsidiaryCode = $params['subsidiaryCode'];
            $storeCode      = $params['storeCode'];
            
            // verifica a existência da filial
            if (!$this->util->subsidiaryExists($subsidiaryCode)) {
                throw new \Exception('Filial inexistente.');
            }
            
            // verifica a existência da loja relacionada à filial
            if (!$this->util->storeExists($subsidiaryCode, $storeCode)) {
                throw new \Exception('Loja inexistente.');
            }
            
            $returnProductsPrinters = $this->menuService->getProductsPrinterParams($subsidiaryCode, $storeCode);
            
            $products = array();
            foreach($returnProductsPrinters['data'] as $productPrinter){
                $product = array(
                    'productCode' => $productPrinter["CDPRODUTO"],
                    'seqPrinterProduct' => $productPrinter["NRSEQIMPRPROD"],
                    'seqPrinterPuxa' => $productPrinter["NRSEQIMPRPUXA"],
                    'seqPrinterProduct2' => $productPrinter["NRSEQIMPRPROD2"]
                );
                $products[] = $product;
            }

            $response->addDataSet(new DataSet('', array('productsPrinterParams' => $products))); 
        } catch (\Exception $e) {
            $message = 'Erro ao buscar os parâmetros de impressora dos produtos. '.$e->getMessage();
            $response->addMessage(new Message($message, self::ERROR_MESSAGE));
        }
    }

    public function V3_getMenuByPosEx(Request $request, Response $response) {
        ini_set('zlib.output_compression_level', 1);
        ob_start('ob_gzhandler');

        $inputJSON = file_get_contents('php://input');
        $input = json_decode($inputJSON, true);

        try {
            if (!isset($input)) {
                throw new \Exception('Não foram recebidos parâmetros');
            }
            
            $data   = array(); // dados a serem retornados

            $params = $input;
            $subsidiaryCode = $params['subsidiaryCode'];
            $storeCode      = $params['storeCode'];
            $posCode        = $params['posCode'];
            $consumerType   = $params['consumerType'];
            $maskedHour     = $params['hour'];
            $hour           = str_replace(':','',$maskedHour);

            if(!empty($params['clientCode'])){
                $clientCode = $params['clientCode'];
            } else{
                $clientCode = null;
            }
            
            // verifica a existência da filial
            if (!$this->util->subsidiaryExists($subsidiaryCode)) {
                throw new \Exception('Filial inexistente.');
            }
            
            // verifica a existência da loja relacionada à filial
            if (!$this->util->storeExists($subsidiaryCode, $storeCode)) {
                throw new \Exception('Loja inexistente.');
            }
            
            // verifica a existência do caixa relacionado à loja
            if (!$this->util->posExists($subsidiaryCode, $storeCode, $posCode)) {
                throw new \Exception('Caixa inexistente.');
            }
            
            $confTelaCaixa = $this->menuService->getConfTelaCaixaEx($subsidiaryCode, $posCode);

            if($confTelaCaixa['error']) {
                $message = $confTelaCaixa['message'];
                $response->addMessage(new Message($message, self::ERROR_MESSAGE));
            }

            $CDFILIALEX = $confTelaCaixa['data']['CDFILIAL'];
            $NRCONFTELA = $confTelaCaixa['data']['NRCONFTELA'];

            $groups = $this->menuService->getGroupsFromConftelaEx($CDFILIALEX, $NRCONFTELA);

            $subGroups = $this->menuService->getSubGroupsFromConftelaEx($CDFILIALEX, $NRCONFTELA);

            $products = $this->menuService->getProductsFromConftelaEx($CDFILIALEX, $NRCONFTELA, $subsidiaryCode);

            $rawAddProducts = $this->menuService->getAddProducts();
          
            $addProducts = [];
            foreach($rawAddProducts['data'] as $addProduct) {
              $addProducts[$addProduct['CDPRODUTO']][] = [
                "isCharged" => $addProduct['IDCOBRAADIC'],
                "addProductCode" => $addProduct['CDSUBPRODUTO'],
                "addProductName" => $addProduct['NMPRODUTO'],
                "saleMode" => $addProduct['IDMODOVENDA'],
                "addProductAmount" => $addProduct['QTPRODADIC']
              ];
            }

            $comboCodes = [];

            foreach($products['data'] as $product) {
                if(!empty($product['CDPRODPROMOCAO'])) {
                    array_push($comboCodes, $product['CDPRODPROMOCAO']);
                }
            }

            $localComboGroups = $this->menuService->getLocalGroupsPromoFromConftelaEx($comboCodes, $subsidiaryCode);
            $comboGroups = $this->menuService->getGroupsPromoFromConftelaEx($comboCodes);

            $comboGroupCodes = [];

            if(!$localComboGroups['error']) {
                foreach($localComboGroups['data'] as $comboGroup) {
                    if(!in_array($comboGroup['CDGRUPROMOC'], $comboGroupCodes))
                        array_push($comboGroupCodes, $comboGroup['CDGRUPROMOC']);
                }
            }

            if(!$comboGroups['error']) {
                foreach($comboGroups['data'] as $comboGroup) {
                    if(!in_array($comboGroup['CDGRUPROMOC'], $comboGroupCodes))
                        array_push($comboGroupCodes, $comboGroup['CDGRUPROMOC']);
                }    
            } 

            if(!empty($comboGroupCodes)) {
              $groupPromoProducts = $this->menuService->getGroupPromoProductsFromConftelaEx($comboGroupCodes, $NRCONFTELA);
            } else {
              $groupPromoProducts = [];
            }

            $productCodes = [];
            
            foreach($products['data'] as $product) {
                if(!in_array($product['CDPRODUTO'], $productCodes))
                    array_push($productCodes, $product['CDPRODUTO']);
            }

            if(!empty($groupPromoProducts['data'])) {
                foreach($groupPromoProducts['data'] as $groupPromoProduct) {
                    if(!in_array($groupPromoProduct['CDPRODUTO'], $productCodes))
                        array_push($productCodes, $groupPromoProduct['CDPRODUTO']);
                }
            }

            if(count($productCodes) >= 2000) {
                $response->addMessage(new Message("Não foi possivel montar o cardapio.", self::ERROR_MESSAGE));
            }
            
            $observations = $this->menuService->getObservationsFromConftelaEx($subsidiaryCode, $storeCode);
            $productPrices = $this->menuService->getProductPrices($subsidiaryCode, $storeCode, $clientCode, $consumerType);
            $comboPrices = $this->menuService->getComboPrices($subsidiaryCode);

            $menu = $this->menuService->buildMenu($groups, $subGroups, $products, $observations, $comboGroups, $localComboGroups, $groupPromoProducts, $productPrices, $comboPrices, $addProducts);
            
            $response->addDataSet(new DataSet('', $menu)); 
        } catch (\Exception $e) {
            $message = 'Erro ao buscar cardápio do caixa. '.$e->getMessage();
            $response->addMessage(new Message($message, self::ERROR_MESSAGE));
        }
    
    }

    public function V3_getMenuByPosExFull(Request $request, Response $response) {
      ini_set('zlib.output_compression_level', 1);
      ob_start('ob_gzhandler');

      $inputJSON = file_get_contents('php://input');
      $input = json_decode($inputJSON, true);

        try {
          if (!isset($input)) {
              throw new \Exception('Não foram recebidos parâmetros');
          }
          
          $data   = array(); // dados a serem retornados

          $params = $input;
          $subsidiaryCode = $params['subsidiaryCode'];
          $storeCode      = $params['storeCode'];
          $posCode        = $params['posCode'];
          $consumerType   = $params['consumerType'];
          $maskedHour     = $params['hour'];
          $hour           = str_replace(':','',$maskedHour);

          if(!empty($params['clientCode'])){
              $clientCode = $params['clientCode'];
          } else{
              $clientCode = null;
          }
          
          // verifica a existência da filial
          if (!$this->util->subsidiaryExists($subsidiaryCode)) {
              throw new \Exception('Filial inexistente.');
          }
          
          // verifica a existência da loja relacionada à filial
          if (!$this->util->storeExists($subsidiaryCode, $storeCode)) {
              throw new \Exception('Loja inexistente.');
          }
          
          // verifica a existência do caixa relacionado à loja
          if (!$this->util->posExists($subsidiaryCode, $storeCode, $posCode)) {
              throw new \Exception('Caixa inexistente.');
          }
          
          $confTelaCaixa = $this->menuService->getConfTelaCaixaEx($subsidiaryCode, $posCode);

          if($confTelaCaixa['error']) {
              $message = $confTelaCaixa['message'];
              $response->addMessage(new Message($message, self::ERROR_MESSAGE));
          }

          $CDFILIALEX = $confTelaCaixa['data']['CDFILIAL'];
          $NRCONFTELA = $confTelaCaixa['data']['NRCONFTELA'];

          $groups = $this->menuService->getGroupsFromConftelaEx($CDFILIALEX, $NRCONFTELA);

          $subGroups = $this->menuService->getSubGroupsFromConftelaEx($CDFILIALEX, $NRCONFTELA);

          $products = $this->menuService->getProductsFromConftelaEx($CDFILIALEX, $NRCONFTELA, $subsidiaryCode);
          
          if($products['error']) {
              throw new \Exception($products['message']);
          }

          $rawAddProducts = $this->menuService->getAddProducts();
          
          $addProducts = [];
          foreach($rawAddProducts['data'] as $addProduct) {
            $addProducts[$addProduct['CDPRODUTO']][] = [
              "isCharged" => $addProduct['IDCOBRAADIC'],
              "addProductCode" => $addProduct['CDSUBPRODUTO'],
              "addProductName" => $addProduct['NMPRODUTO'],
              "saleMode" => $addProduct['IDMODOVENDA'],
              "addProductAmount" => $addProduct['QTPRODADIC']
            ];
          }

          $comboCodes = [];

          foreach($products['data'] as $product) {
              if(!empty($product['CDPRODPROMOCAO'])) {
                  array_push($comboCodes, $product['CDPRODPROMOCAO']);
              }
          }
          
          $removeProductsGroup = $this->menuService->removeProductsWithoutPrice($subsidiaryCode);
          $removeProducts = $removeProductsGroup['IDEXIBEPRECZERO'] === 'N' ? true : false;
          
          if(isset($removeProducts) && $removeProducts){
            $localComboGroups = $this->menuService->getLocalGroupsPromoFromConftelaExWithPrice($comboCodes, $subsidiaryCode);
            $comboGroups = $this->menuService->getGroupsPromoFromConftelaExWithPrice($comboCodes, $subsidiaryCode);
          } else{
            $localComboGroups = $this->menuService->getLocalGroupsPromoFromConftelaEx($comboCodes, $subsidiaryCode);
            $comboGroups = $this->menuService->getGroupsPromoFromConftelaEx($comboCodes);
          }

          $comboGroupCodes = [];

          if(!$localComboGroups['error']) {
              foreach($localComboGroups['data'] as $comboGroup) {
                  if(!in_array($comboGroup['CDGRUPROMOC'], $comboGroupCodes))
                      array_push($comboGroupCodes, $comboGroup['CDGRUPROMOC']);
              }
          }
          
          if(!$comboGroups['error']) {
              foreach($comboGroups['data'] as $comboGroup) {
                  if(!in_array($comboGroup['CDGRUPROMOC'], $comboGroupCodes))
                      array_push($comboGroupCodes, $comboGroup['CDGRUPROMOC']);
              }    
          } 

          if(!empty($comboGroupCodes)) {
            if(isset($removeProducts) && $removeProducts) {
              $groupPromoProducts = $this->menuService->getGroupPromoProductsFromConftelaExWithPrice($comboGroupCodes, $NRCONFTELA, $subsidiaryCode);    
            } else {
              $groupPromoProducts = $this->menuService->getGroupPromoProductsFromConftelaEx($comboGroupCodes, $NRCONFTELA);
            }
          } else {
            $groupPromoProducts = [];
          }

            $productCodes = [];
          
          foreach($products['data'] as $product) {
              if(!in_array($product['CDPRODUTO'], $productCodes))
                  array_push($productCodes, $product['CDPRODUTO']);
          }

          if(!empty($groupPromoProducts['data'])) {
              foreach($groupPromoProducts['data'] as $groupPromoProduct) {
                  if(!in_array($groupPromoProduct['CDPRODUTO'], $productCodes))
                      array_push($productCodes, $groupPromoProduct['CDPRODUTO']);
              }
          }

          if(count($productCodes) >= 2000) {
              $response->addMessage(new Message("Não foi possivel montar o cardapio.", self::ERROR_MESSAGE));
          }
          
          $observations = $this->menuService->getObservationsFromConftelaEx($subsidiaryCode, $storeCode);
          $productPrices = $this->menuService->getProductPrices($subsidiaryCode, $storeCode, $clientCode, $consumerType);
          $comboPrices = $this->menuService->getComboPrices($subsidiaryCode);
          $famSaldProds = $this->menuService->getFamSaldProds($subsidiaryCode);

          $menu = $this->menuService->buildMenu($groups, $subGroups, $products, $observations, $comboGroups, $localComboGroups, $groupPromoProducts, $productPrices, $comboPrices, $addProducts, $famSaldProds);

          foreach ($menu['groups'] as &$group) {
            if(!empty($group['subGroups'])){
              $group['products'] = [];
            } else{
              foreach ($group['products'] as $key => &$product){
                // Base price (for products and combos)
                if($product['productPrint'] === 'N' && !empty($product['productGroups'])){
                  $product['fullPrice'] = 0.0;
                  $product['price'] = 0.0;
                  $product['discount'] = 0.0;
                  $product['addition'] = 0.0;
                } else{
                  $product['fullPrice'] = (!empty($menu['prices']['prices'][$product['productCode']]['PRECO']) ? $menu['prices']['prices'][$product['productCode']]['PRECO'] : 0.01);
                  $product['price'] = $product['fullPrice'];
  
                  //Daily price
                  if(!empty($menu['prices']['dailyPrices'][$product['productCode']])){
                    if($menu['prices']['dailyPrices'][$product['productCode']]['IDDESCACREPR'] === 'D'){
                      if($menu['prices']['dailyPrices'][$product['productCode']]['IDPERVALORPR'] === 'V'){
                        $product['price'] -= $menu['prices']['dailyPrices'][$product['productCode']]['VRPRECODIA'];
                      } else{
                        $product['price'] -= ($product['price'] * ($menu['prices']['dailyPrices'][$product['productCode']]['VRPRECODIA'] / 100));
                      }
                    } else{
                      if($menu['prices']['dailyPrices'][$product['productCode']]['IDPERVALORPR'] === 'V'){
                        $product['price'] += $menu['prices']['dailyPrices'][$product['productCode']]['VRPRECODIA'];
                      } else{
                        $product['price'] += ($product['price'] * ($menu['prices']['dailyPrices'][$product['productCode']]['VRPRECODIA'] / 100));
                      }
                    }
                  }
                  
                  $product['price'] = floatval(bcadd((string)$product['price'], '0.00', 2));
                  $product['discount'] = floatval(bcsub((string)$product['fullPrice'], $product['price'], 2));
                  if($product['discount'] < 0.00){
                    $product['addition'] = -1 * $product['discount'];
                    $product['discount'] = 0.0;
                  } else{
                    $product['addition'] = 0.0;
                  }

                  if($product['fullPrice'] === 0.0){
                    unset($group['products'][$key]);
                    continue;
                  }
                }

                //Observations price
                foreach ($product['observationsChoice'] as &$observationGroup){
                  foreach($observationGroup['observations'] as &$observationChoice){
                    if($observationChoice['obsType'] === 'A') {
                      $observationChoice['fullPrice'] = (!empty($menu['prices']['prices'][$observationChoice['obsProductCode']]['PRECO']) ? $menu['prices']['prices'][$observationChoice['obsProductCode']]['PRECO'] : 0.01);
                      $observationChoice['price'] = $observationChoice['fullPrice'];
    
                      //Daily price
                      if(!empty($menu['prices']['dailyPrices'][$observationChoice['obsProductCode']])){
                        if($menu['prices']['dailyPrices'][$observationChoice['obsProductCode']]['IDDESCACREPR'] === 'D'){
                          if($menu['prices']['dailyPrices'][$observationChoice['obsProductCode']]['IDPERVALORPR'] === 'V'){
                            $observationChoice['price'] -= $menu['prices']['dailyPrices'][$observationChoice['obsProductCode']]['VRPRECODIA'];
                          } else{
                            $observationChoice['price'] -= ($observationChoice['price'] * ($menu['prices']['dailyPrices'][$observationChoice['obsProductCode']]['VRPRECODIA'] / 100));
                          }
                        } else{
                          if($menu['prices']['dailyPrices'][$observationChoice['obsProductCode']]['IDPERVALORPR'] === 'V'){
                            $observationChoice['price'] += $menu['prices']['dailyPrices'][$observationChoice['obsProductCode']]['VRPRECODIA'];
                          } else{
                            $observationChoice['price'] += ($observationChoice['price'] * ($menu['prices']['dailyPrices'][$observationChoice['obsProductCode']]['VRPRECODIA'] / 100));
                          }
                        }
                      }
    
                      $observationChoice['price'] = floatval(bcadd((string)$observationChoice['price'], '0.00', 2));
                      $observationChoice['discount'] = floatval(bcsub((string)$observationChoice['fullPrice'], $observationChoice['price'], 2));
                      if($observationChoice['discount'] < 0.00){
                        $observationChoice['addition'] = -1 * $observationChoice['discount'];
                        $observationChoice['discount'] = 0.0;
                      } else{
                        $observationChoice['addition'] = 0.0;
                      }
                    } else {
                      $observationChoice['fullPrice'] = 0.0;
                      $observationChoice['price'] = 0.0;
                      $observationChoice['discount'] = 0.0;
                      $observationChoice['addition'] = 0.0;
                    }
                  }
                }
  
                if(isset($observationChoice)) 
                    unset($observationChoice);
  
                foreach ($product['productGroups'] as &$productGroup) {
                  $productGroup['productGroupName'] = $menu['combos'][$productGroup['productGroupCode']]['comboGroupName'];
                  $productGroup['productGroupItems'] = $menu['combos'][$productGroup['productGroupCode']]['comboGroupItems'];

                  foreach ($productGroup['productGroupItems'] as $comboKey => &$comboGroupItem){
                    
                    //Combo items price
                    if(!empty($comboGroupItem['subItemProductGroups']) && $comboGroupItem['subItemProductPrint'] == 'N'){
                      $comboGroupItem['fullPrice'] = 0.0;
                      $comboGroupItem['price'] = 0.0;
                      $comboGroupItem['discount'] = 0.0;
                      $comboGroupItem['addition'] = 0.0;
                    } else{
                      $comboGroupItem['fullPrice'] = (!empty($menu['prices']['prices'][$comboGroupItem['subItemCode']]['PRECO']) ? $menu['prices']['prices'][$comboGroupItem['subItemCode']]['PRECO'] : 0.01);
                      $comboGroupItem['price'] = $comboGroupItem['fullPrice'];
  
                      //Combo diff price
                      if(!empty($menu['prices']['comboDiffPrices'][$comboGroupItem['subItemCode']][$productGroup['productGroupCode']][$product['productCode']])){
                        if($menu['prices']['comboDiffPrices'][$comboGroupItem['subItemCode']][$productGroup['productGroupCode']][$product['productCode']]['IDDESCACRPROMO'] === 'D'){
                          if($menu['prices']['comboDiffPrices'][$comboGroupItem['subItemCode']][$productGroup['productGroupCode']][$product['productCode']]['IDPERCVALOR'] === 'V'){
                            $comboGroupItem['price'] -= $menu['prices']['comboDiffPrices'][$comboGroupItem['subItemCode']][$productGroup['productGroupCode']][$product['productCode']]['VRDESCACR'];
                          } else{
                            $comboGroupItem['price'] -= ($comboGroupItem['price'] * (($menu['prices']['comboDiffPrices'][$comboGroupItem['subItemCode']][$productGroup['productGroupCode']][$product['productCode']]['VRDESCACR']) / 100));
                          }
                        } else{
                          if($menu['prices']['comboDiffPrices'][$comboGroupItem['subItemCode']][$productGroup['productGroupCode']][$product['productCode']]['IDPERCVALOR'] === 'V'){
                            $comboGroupItem['price'] += $menu['prices']['comboDiffPrices'][$comboGroupItem['subItemCode']][$productGroup['productGroupCode']][$product['productCode']]['VRDESCACR'];
                          } else{
                            $comboGroupItem['price'] += ($comboGroupItem['price'] * (($menu['prices']['comboDiffPrices'][$comboGroupItem['subItemCode']][$productGroup['productGroupCode']][$product['productCode']]['VRDESCACR']) / 100));
                          }
                        }
                      }
  
                      //Daily price
                      if(!empty($menu['prices']['dailyPrices'][$comboGroupItem['subItemCode']])){
                        if($menu['prices']['dailyPrices'][$comboGroupItem['subItemCode']]['IDDESCACREPR'] === 'D'){
                          if($menu['prices']['dailyPrices'][$comboGroupItem['subItemCode']]['IDPERVALORPR'] === 'V'){
                            $comboGroupItem['price'] -= $menu['prices']['dailyPrices'][$comboGroupItem['subItemCode']]['VRPRECODIA'];
                          } else{
                            $comboGroupItem['price'] -= ($comboGroupItem['price'] * ($menu['prices']['dailyPrices'][$comboGroupItem['subItemCode']]['VRPRECODIA'] / 100));
                          }
                        } else{
                          if($menu['prices']['dailyPrices'][$comboGroupItem['subItemCode']]['IDPERVALORPR'] === 'V'){
                            $comboGroupItem['price'] += $menu['prices']['dailyPrices'][$comboGroupItem['subItemCode']]['VRPRECODIA'];
                          } else{
                            $comboGroupItem['price'] += ($comboGroupItem['price'] * ($menu['prices']['dailyPrices'][$comboGroupItem['subItemCode']]['VRPRECODIA'] / 100));
                          }
                        }
                      }
  
                      $comboGroupItem['price'] = floatval(bcadd((string)$comboGroupItem['price'], '0.00', 2));
                      $comboGroupItem['discount'] = floatval(bcsub((string)$comboGroupItem['fullPrice'], $comboGroupItem['price'], 2));
                      if($comboGroupItem['discount'] < 0.00){
                        $comboGroupItem['addition'] = -1 * $comboGroupItem['discount'];
                        $comboGroupItem['discount'] = 0.0;
                      } else{
                        $comboGroupItem['addition'] = 0.0;
                      }

                      if($comboGroupItem['fullPrice'] === 0.0){
                        unset($productGroup['productGroupItems'][$comboKey]);
                        continue;
                      }
                    }
  
                    //Combo item obs price
                    foreach ($comboGroupItem['subItemObservationsChoice'] as &$cbItmObservationGroup){
                      foreach($cbItmObservationGroup['subItemObservations'] as &$cbItmObservationChoice){
                        if($cbItmObservationChoice['subItemObsType'] === 'A') {
                          $cbItmObservationChoice['fullPrice'] = (!empty($menu['prices']['prices'][$cbItmObservationChoice['subItemObsProductCode']]['PRECO']) ? $menu['prices']['prices'][$cbItmObservationChoice['subItemObsProductCode']]['PRECO'] : 0.01);
                          $cbItmObservationChoice['price'] = $cbItmObservationChoice['fullPrice'];
    
                          //Daily price
                          if(!empty($menu['prices']['dailyPrices'][$cbItmObservationChoice['subItemObsProductCode']])){
                            if($menu['prices']['dailyPrices'][$cbItmObservationChoice['subItemObsProductCode']]['IDDESCACREPR'] === 'D'){
                              if($menu['prices']['dailyPrices'][$cbItmObservationChoice['subItemObsProductCode']]['IDPERVALORPR'] === 'V'){
                                $cbItmObservationChoice['price'] -= $menu['prices']['dailyPrices'][$cbItmObservationChoice['subItemObsProductCode']]['VRPRECODIA'];
                              } else{
                                $cbItmObservationChoice['price'] -= ($cbItmObservationChoice['price'] * ($menu['prices']['dailyPrices'][$cbItmObservationChoice['subItemObsProductCode']]['VRPRECODIA'] / 100));
                              }
                            } else{
                              if($menu['prices']['dailyPrices'][$cbItmObservationChoice['subItemObsProductCode']]['IDPERVALORPR'] === 'V'){
                                $cbItmObservationChoice['price'] += $menu['prices']['dailyPrices'][$cbItmObservationChoice['subItemObsProductCode']]['VRPRECODIA'];
                              } else{
                                $cbItmObservationChoice['price'] += ($cbItmObservationChoice['price'] * ($menu['prices']['dailyPrices'][$cbItmObservationChoice['subItemObsProductCode']]['VRPRECODIA'] / 100));
                              }
                            }
                          }
    
                          $cbItmObservationChoice['price'] = floatval(bcadd((string)$cbItmObservationChoice['price'], '0.00', 2));
                          $cbItmObservationChoice['discount'] = floatval(bcsub((string)$cbItmObservationChoice['fullPrice'], $cbItmObservationChoice['price'], 2));
                          if($cbItmObservationChoice['discount'] < 0.00){
                            $cbItmObservationChoice['addition'] = -1 * $cbItmObservationChoice['discount'];
                            $cbItmObservationChoice['discount'] = 0.0;
                          } else{
                            $cbItmObservationChoice['addition'] = 0.0;
                          }
                        } else {
                          $cbItmObservationChoice['fullPrice'] = 0.0;
                          $cbItmObservationChoice['price'] = 0.0;
                          $cbItmObservationChoice['discount'] = 0.0;
                          $cbItmObservationChoice['addition'] = 0.0;
                        }
                      }
                    }
  
                    if(isset($cbItmObservationChoice)) unset($cbItmObservationChoice);
  
                    foreach ($comboGroupItem['subItemProductGroups'] as &$subItemProductGroup){
                      $subItemProductGroup['subProductGroupName'] = $menu['combos'][$subItemProductGroup['subProductGroupCode']]['comboGroupName'];
                      $subItemProductGroup['subProductGroupItems'] = $menu['combos'][$subItemProductGroup['subProductGroupCode']]['comboGroupItems'];

                      //Combo inside combo price
                      foreach ($subItemProductGroup['subProductGroupItems'] as $subComboKey => &$subComboGroupItem){
                        $subComboGroupItem['subItemProductGroups'] = [];
                        $subComboGroupItem['fullPrice'] = (!empty($menu['prices']['prices'][$subComboGroupItem['subItemCode']]['PRECO']) ? $menu['prices']['prices'][$subComboGroupItem['subItemCode']]['PRECO'] : 0.01);
                        $subComboGroupItem['price'] = $subComboGroupItem['fullPrice'];
  
                        //Combo diff price
                        if(!empty($menu['prices']['comboDiffPrices'][$subComboGroupItem['subItemCode']][$subItemProductGroup['subProductGroupCode']][$comboGroupItem['subItemCode']])){
                          if($menu['prices']['comboDiffPrices'][$subComboGroupItem['subItemCode']][$subItemProductGroup['subProductGroupCode']][$comboGroupItem['subItemCode']]['IDDESCACRPROMO'] === 'D'){
                            if($menu['prices']['comboDiffPrices'][$subComboGroupItem['subItemCode']][$subItemProductGroup['subProductGroupCode']][$comboGroupItem['subItemCode']]['IDPERCVALOR'] === 'V'){
                              $subComboGroupItem['price'] -= $menu['prices']['comboDiffPrices'][$subComboGroupItem['subItemCode']][$subItemProductGroup['subProductGroupCode']][$comboGroupItem['subItemCode']]['VRDESCACR'];
                            } else{
                              $subComboGroupItem['price'] -= ($subComboGroupItem['price'] * (($menu['prices']['comboDiffPrices'][$subComboGroupItem['subItemCode']][$subItemProductGroup['subProductGroupCode']][$comboGroupItem['subItemCode']]['VRDESCACR']) / 100));
                            }
                          } else{
                            if($menu['prices']['comboDiffPrices'][$subComboGroupItem['subItemCode']][$subItemProductGroup['subProductGroupCode']][$comboGroupItem['subItemCode']]['IDPERCVALOR'] === 'V'){
                              $subComboGroupItem['price'] += $menu['prices']['comboDiffPrices'][$subComboGroupItem['subItemCode']][$subItemProductGroup['subProductGroupCode']][$comboGroupItem['subItemCode']]['VRDESCACR'];
                            } else{
                              $subComboGroupItem['price'] += ($subComboGroupItem['price'] * (($menu['prices']['comboDiffPrices'][$subComboGroupItem['subItemCode']][$subItemProductGroup['subProductGroupCode']][$comboGroupItem['subItemCode']]['VRDESCACR']) / 100));
                            }
                          }
                        }
  
                        //Daily price
                        if(!empty($menu['prices']['dailyPrices'][$subComboGroupItem['subItemCode']])){
                          if($menu['prices']['dailyPrices'][$subComboGroupItem['subItemCode']]['IDDESCACREPR'] === 'D'){
                            if($menu['prices']['dailyPrices'][$subComboGroupItem['subItemCode']]['IDPERVALORPR'] === 'V'){
                              $subComboGroupItem['price'] -= $menu['prices']['dailyPrices'][$subComboGroupItem['subItemCode']]['VRPRECODIA'];
                            } else{
                              $subComboGroupItem['price'] -= ($subComboGroupItem['price'] * ($menu['prices']['dailyPrices'][$subComboGroupItem['subItemCode']]['VRPRECODIA'] / 100));
                            }
                          } else{
                            if($menu['prices']['dailyPrices'][$subComboGroupItem['subItemCode']]['IDPERVALORPR'] === 'V'){
                              $subComboGroupItem['price'] += $menu['prices']['dailyPrices'][$subComboGroupItem['subItemCode']]['VRPRECODIA'];
                            } else{
                              $subComboGroupItem['price'] += ($subComboGroupItem['price'] * ($menu['prices']['dailyPrices'][$subComboGroupItem['subItemCode']]['VRPRECODIA'] / 100));
                            }
                          }
                        }
  
                        $subComboGroupItem['price'] = floatval(bcadd((string)$subComboGroupItem['price'], '0.00', 2));
                        $subComboGroupItem['discount'] = floatval(bcsub((string)$subComboGroupItem['fullPrice'], $subComboGroupItem['price'], 2));
                        if($subComboGroupItem['discount'] < 0.00){
                          $subComboGroupItem['addition'] = -1 * $subComboGroupItem['discount'];
                          $subComboGroupItem['discount'] = 0.0;
                        } else{
                          $subComboGroupItem['addition'] = 0.0;
                        }

                        if($subComboGroupItem['fullPrice'] === 0.0){
                          unset($subItemProductGroup['subProductGroupItems'][$subComboKey]);
                          continue;
                        }
  
                        //Combo item obs price
                        foreach ($subComboGroupItem['subItemObservationsChoice'] as &$sbCbItmObservationGroup){
                          foreach($sbCbItmObservationGroup['subItemObservations'] as &$sbCbItmObservationChoice){
                            if($sbCbItmObservationChoice['subItemObsType'] === 'A') {
                              $sbCbItmObservationChoice['fullPrice'] = (!empty($menu['prices']['prices'][$sbCbItmObservationChoice['subItemObsProductCode']]['PRECO']) ? $menu['prices']['prices'][$sbCbItmObservationChoice['subItemObsProductCode']]['PRECO'] : 0.01);
                              $sbCbItmObservationChoice['price'] = $sbCbItmObservationChoice['fullPrice'];
    
                              //Daily price
                              if(!empty($menu['prices']['dailyPrices'][$sbCbItmObservationChoice['subItemObsProductCode']])){
                                if($menu['prices']['dailyPrices'][$sbCbItmObservationChoice['subItemObsProductCode']]['IDDESCACREPR'] === 'D'){
                                  if($menu['prices']['dailyPrices'][$sbCbItmObservationChoice['subItemObsProductCode']]['IDPERVALORPR'] === 'V'){
                                    $sbCbItmObservationChoice['price'] -= $menu['prices']['dailyPrices'][$sbCbItmObservationChoice['subItemObsProductCode']]['VRPRECODIA'];
                                  } else{
                                    $sbCbItmObservationChoice['price'] -= ($sbCbItmObservationChoice['price'] * ($menu['prices']['dailyPrices'][$sbCbItmObservationChoice['subItemObsProductCode']]['VRPRECODIA'] / 100));
                                  }
                                } else{
                                  if($menu['prices']['dailyPrices'][$sbCbItmObservationChoice['subItemObsProductCode']]['IDPERVALORPR'] === 'V'){
                                    $sbCbItmObservationChoice['price'] += $menu['prices']['dailyPrices'][$sbCbItmObservationChoice['subItemObsProductCode']]['VRPRECODIA'];
                                  } else{
                                    $sbCbItmObservationChoice['price'] += ($sbCbItmObservationChoice['price'] * ($menu['prices']['dailyPrices'][$sbCbItmObservationChoice['subItemObsProductCode']]['VRPRECODIA'] / 100));
                                  }
                                }
                              }
            
                              $sbCbItmObservationChoice['price'] = floatval(bcadd((string)$sbCbItmObservationChoice['price'], '0.00', 2));
                              $sbCbItmObservationChoice['discount'] = floatval(bcsub((string)$sbCbItmObservationChoice['fullPrice'], $sbCbItmObservationChoice['price'], 2));
                              if($sbCbItmObservationChoice['discount'] < 0.00){
                                $sbCbItmObservationChoice['addition'] = -1 * $sbCbItmObservationChoice['discount'];
                                $sbCbItmObservationChoice['discount'] = 0.0;
                              } else{
                                $sbCbItmObservationChoice['addition'] = 0.0;
                              }
                            } else {
                              $sbCbItmObservationChoice['fullPrice'] = 0.0;
                              $sbCbItmObservationChoice['price'] = 0.0;
                              $sbCbItmObservationChoice['discount'] = 0.0;
                              $sbCbItmObservationChoice['addition'] = 0.0;
                            }
                          }
                        }
  
                        if(isset($sbCbItmObservationChoice)) unset($sbCbItmObservationChoice);
  
                      }
  
                      if(isset($subComboGroupItem)) unset($subComboGroupItem);
                    }
  
                    if(isset($subItemProductGroup)) unset($subItemProductGroup);
                  }
  
                  if(isset($comboGroupItem)) unset($comboGroupItem);
                }
  
                if(isset($productGroup)) unset($productGroup);
              }
            }

            if(isset($product)) unset($product);

            foreach ($group['subGroups'] as &$subGroup){
              foreach ($subGroup['products'] as $key => &$product){
                // Base price (for products and combos)
                if($product['productPrint'] === 'N' && !empty($product['productGroups'])){
                  $product['fullPrice'] = 0.0;
                  $product['price'] = 0.0;
                  $product['discount'] = 0.0;
                  $product['addition'] = 0.0;
                } else{
                  $product['fullPrice'] = (!empty($menu['prices']['prices'][$product['productCode']]['PRECO']) ? $menu['prices']['prices'][$product['productCode']]['PRECO'] : 0.01);
                  $product['price'] = $product['fullPrice'];
  
                  //Daily price
                  if(!empty($menu['prices']['dailyPrices'][$product['productCode']])){
                    if($menu['prices']['dailyPrices'][$product['productCode']]['IDDESCACREPR'] === 'D'){
                      if($menu['prices']['dailyPrices'][$product['productCode']]['IDPERVALORPR'] === 'V'){
                        $product['price'] -= $menu['prices']['dailyPrices'][$product['productCode']]['VRPRECODIA'];
                      } else{
                        $product['price'] -= ($product['price'] * ($menu['prices']['dailyPrices'][$product['productCode']]['VRPRECODIA'] / 100));
                      }
                    } else{
                      if($menu['prices']['dailyPrices'][$product['productCode']]['IDPERVALORPR'] === 'V'){
                        $product['price'] += $menu['prices']['dailyPrices'][$product['productCode']]['VRPRECODIA'];
                      } else{
                        $product['price'] += ($product['price'] * ($menu['prices']['dailyPrices'][$product['productCode']]['VRPRECODIA'] / 100));
                      }
                    }
                  }
                  
                  $product['price'] = floatval(bcadd((string)$product['price'], '0.00', 2));
                  $product['discount'] = floatval(bcsub((string)$product['fullPrice'], $product['price'], 2));
                  if($product['discount'] < 0.00){
                    $product['addition'] = -1 * $product['discount'];
                    $product['discount'] = 0.0;
                  } else{
                    $product['addition'] = 0.0;
                  }

                  if($product['fullPrice'] === 0.0){
                    unset($subGroup['products'][$key]);
                    continue;
                  }
                }
  
                //Observations price
                foreach ($product['observationsChoice'] as &$observationGroup){
                  foreach($observationGroup['observations'] as &$observationChoice){
                    if($observationChoice['obsType'] === 'A') {
                      $observationChoice['fullPrice'] = (!empty($menu['prices']['prices'][$observationChoice['obsProductCode']]['PRECO']) ? $menu['prices']['prices'][$observationChoice['obsProductCode']]['PRECO'] : 0.01);
                      $observationChoice['price'] = $observationChoice['fullPrice'];
    
                      //Daily price
                      if(!empty($menu['prices']['dailyPrices'][$observationChoice['obsProductCode']])){
                        if($menu['prices']['dailyPrices'][$observationChoice['obsProductCode']]['IDDESCACREPR'] === 'D'){
                          if($menu['prices']['dailyPrices'][$observationChoice['obsProductCode']]['IDPERVALORPR'] === 'V'){
                            $observationChoice['price'] -= $menu['prices']['dailyPrices'][$observationChoice['obsProductCode']]['VRPRECODIA'];
                          } else{
                            $observationChoice['price'] -= ($observationChoice['price'] * ($menu['prices']['dailyPrices'][$observationChoice['obsProductCode']]['VRPRECODIA'] / 100));
                          }
                        } else{
                          if($menu['prices']['dailyPrices'][$observationChoice['obsProductCode']]['IDPERVALORPR'] === 'V'){
                            $observationChoice['price'] += $menu['prices']['dailyPrices'][$observationChoice['obsProductCode']]['VRPRECODIA'];
                          } else{
                            $observationChoice['price'] += ($observationChoice['price'] * ($menu['prices']['dailyPrices'][$observationChoice['obsProductCode']]['VRPRECODIA'] / 100));
                          }
                        }
                      }
    
                      $observationChoice['price'] = floatval(bcadd((string)$observationChoice['price'], '0.00', 2));
                      $observationChoice['discount'] = floatval(bcsub((string)$observationChoice['fullPrice'], $observationChoice['price'], 2));
                      if($observationChoice['discount'] < 0.00){
                        $observationChoice['addition'] = -1 * $observationChoice['discount'];
                        $observationChoice['discount'] = 0.0;
                      } else{
                        $observationChoice['addition'] = 0.0;
                      }
                    } else {
                      $observationChoice['fullPrice'] = 0.0;
                      $observationChoice['price'] = 0.0;
                      $observationChoice['discount'] = 0.0;
                      $observationChoice['addition'] = 0.0;
                    }
                  }
                }
  
                if(isset($observationChoice)) unset($observationChoice);
  
                foreach ($product['productGroups'] as &$productGroup) {
                  $productGroup['productGroupName'] = $menu['combos'][$productGroup['productGroupCode']]['comboGroupName'];
                  $productGroup['productGroupItems'] = $menu['combos'][$productGroup['productGroupCode']]['comboGroupItems'];
  
                  foreach ($productGroup['productGroupItems'] as $comboKey => &$comboGroupItem){
                    
                    //Combo items price
                    if(!empty($comboGroupItem['subItemProductGroups']) && $comboGroupItem['subItemProductPrint'] == 'N'){
                      $comboGroupItem['fullPrice'] = 0.0;
                      $comboGroupItem['price'] = 0.0;
                      $comboGroupItem['discount'] = 0.0;
                      $comboGroupItem['addition'] = 0.0;
                    } else{
                      $comboGroupItem['fullPrice'] = (!empty($menu['prices']['prices'][$comboGroupItem['subItemCode']]['PRECO']) ? $menu['prices']['prices'][$comboGroupItem['subItemCode']]['PRECO'] : 0.01);
                      $comboGroupItem['price'] = $comboGroupItem['fullPrice'];
  
                      //Combo diff price
                      if(!empty($menu['prices']['comboDiffPrices'][$comboGroupItem['subItemCode']][$productGroup['productGroupCode']][$product['productCode']])){
                        if($menu['prices']['comboDiffPrices'][$comboGroupItem['subItemCode']][$productGroup['productGroupCode']][$product['productCode']]['IDDESCACRPROMO'] === 'D'){
                          if($menu['prices']['comboDiffPrices'][$comboGroupItem['subItemCode']][$productGroup['productGroupCode']][$product['productCode']]['IDPERCVALOR'] === 'V'){
                            $comboGroupItem['price'] -= $menu['prices']['comboDiffPrices'][$comboGroupItem['subItemCode']][$productGroup['productGroupCode']][$product['productCode']]['VRDESCACR'];
                          } else{
                            $comboGroupItem['price'] -= ($comboGroupItem['price'] * (($menu['prices']['comboDiffPrices'][$comboGroupItem['subItemCode']][$productGroup['productGroupCode']][$product['productCode']]['VRDESCACR']) / 100));
                          }
                        } else{
                          if($menu['prices']['comboDiffPrices'][$comboGroupItem['subItemCode']][$productGroup['productGroupCode']][$product['productCode']]['IDPERCVALOR'] === 'V'){
                            $comboGroupItem['price'] += $menu['prices']['comboDiffPrices'][$comboGroupItem['subItemCode']][$productGroup['productGroupCode']][$product['productCode']]['VRDESCACR'];
                          } else{
                            $comboGroupItem['price'] += ($comboGroupItem['price'] * (($menu['prices']['comboDiffPrices'][$comboGroupItem['subItemCode']][$productGroup['productGroupCode']][$product['productCode']]['VRDESCACR']) / 100));
                          }
                        }
                      }
  
                      //Daily price
                      if(!empty($menu['prices']['dailyPrices'][$comboGroupItem['subItemCode']])){
                        if($menu['prices']['dailyPrices'][$comboGroupItem['subItemCode']]['IDDESCACREPR'] === 'D'){
                          if($menu['prices']['dailyPrices'][$comboGroupItem['subItemCode']]['IDPERVALORPR'] === 'V'){
                            $comboGroupItem['price'] -= $menu['prices']['dailyPrices'][$comboGroupItem['subItemCode']]['VRPRECODIA'];
                          } else{
                            $comboGroupItem['price'] -= ($comboGroupItem['price'] * ($menu['prices']['dailyPrices'][$comboGroupItem['subItemCode']]['VRPRECODIA'] / 100));
                          }
                        } else{
                          if($menu['prices']['dailyPrices'][$comboGroupItem['subItemCode']]['IDPERVALORPR'] === 'V'){
                            $comboGroupItem['price'] += $menu['prices']['dailyPrices'][$comboGroupItem['subItemCode']]['VRPRECODIA'];
                          } else{
                            $comboGroupItem['price'] += ($comboGroupItem['price'] * ($menu['prices']['dailyPrices'][$comboGroupItem['subItemCode']]['VRPRECODIA'] / 100));
                          }
                        }
                      }
  
                      $comboGroupItem['price'] = floatval(bcadd((string)$comboGroupItem['price'], '0.00', 2));
                      $comboGroupItem['discount'] = floatval(bcsub((string)$comboGroupItem['fullPrice'], $comboGroupItem['price'], 2));
                      if($comboGroupItem['discount'] < 0.00){
                        $comboGroupItem['addition'] = -1 * $comboGroupItem['discount'];
                        $comboGroupItem['discount'] = 0.0;
                      } else{
                        $comboGroupItem['addition'] = 0.0;
                      }

                      if($comboGroupItem['fullPrice'] === 0.0){
                        unset($productGroup['productGroupItems'][$comboKey]);
                        continue;
                      }
                    }
  
                    //Combo item obs price
                    foreach ($comboGroupItem['subItemObservationsChoice'] as &$cbItmObservationGroup){
                      foreach($cbItmObservationGroup['subItemObservations'] as &$cbItmObservationChoice){
                        if($cbItmObservationChoice['subItemObsType'] === 'A') {
                          $cbItmObservationChoice['fullPrice'] = (!empty($menu['prices']['prices'][$cbItmObservationChoice['subItemObsProductCode']]['PRECO']) ? $menu['prices']['prices'][$cbItmObservationChoice['subItemObsProductCode']]['PRECO'] : 0.01);
                          $cbItmObservationChoice['price'] = $cbItmObservationChoice['fullPrice'];
    
                          //Daily price
                          if(!empty($menu['prices']['dailyPrices'][$cbItmObservationChoice['subItemObsProductCode']])){
                            if($menu['prices']['dailyPrices'][$cbItmObservationChoice['subItemObsProductCode']]['IDDESCACREPR'] === 'D'){
                              if($menu['prices']['dailyPrices'][$cbItmObservationChoice['subItemObsProductCode']]['IDPERVALORPR'] === 'V'){
                                $cbItmObservationChoice['price'] -= $menu['prices']['dailyPrices'][$cbItmObservationChoice['subItemObsProductCode']]['VRPRECODIA'];
                              } else{
                                $cbItmObservationChoice['price'] -= ($cbItmObservationChoice['price'] * ($menu['prices']['dailyPrices'][$cbItmObservationChoice['subItemObsProductCode']]['VRPRECODIA'] / 100));
                              }
                            } else{
                              if($menu['prices']['dailyPrices'][$cbItmObservationChoice['subItemObsProductCode']]['IDPERVALORPR'] === 'V'){
                                $cbItmObservationChoice['price'] += $menu['prices']['dailyPrices'][$cbItmObservationChoice['subItemObsProductCode']]['VRPRECODIA'];
                              } else{
                                $cbItmObservationChoice['price'] += ($cbItmObservationChoice['price'] * ($menu['prices']['dailyPrices'][$cbItmObservationChoice['subItemObsProductCode']]['VRPRECODIA'] / 100));
                              }
                            }
                          }
    
                          $cbItmObservationChoice['price'] = floatval(bcadd((string)$cbItmObservationChoice['price'], '0.00', 2));
                          $cbItmObservationChoice['discount'] = floatval(bcsub((string)$cbItmObservationChoice['fullPrice'], $cbItmObservationChoice['price'], 2));
                          if($cbItmObservationChoice['discount'] < 0.00){
                            $cbItmObservationChoice['addition'] = -1 * $cbItmObservationChoice['discount'];
                            $cbItmObservationChoice['discount'] = 0.0;
                          } else{
                            $cbItmObservationChoice['addition'] = 0.0;
                          }
                        } else {
                          $cbItmObservationChoice['fullPrice'] = 0.0;
                          $cbItmObservationChoice['price'] = 0.0;
                          $cbItmObservationChoice['discount'] = 0.0;
                          $cbItmObservationChoice['addition'] = 0.0;
                        }
                      }
                    }
  
                    if(isset($cbItmObservationChoice)) unset($cbItmObservationChoice);
  
                    foreach ($comboGroupItem['subItemProductGroups'] as &$subItemProductGroup){
                      $subItemProductGroup['subProductGroupName'] = $menu['combos'][$subItemProductGroup['subProductGroupCode']]['comboGroupName'];
                      $subItemProductGroup['subProductGroupItems'] = $menu['combos'][$subItemProductGroup['subProductGroupCode']]['comboGroupItems'];
                      
                      //Combo inside combo price
                      foreach ($subItemProductGroup['subProductGroupItems'] as $subComboKey => &$subComboGroupItem){
                        $subComboGroupItem['subItemProductGroups'] = [];
                        $subComboGroupItem['fullPrice'] = (!empty($menu['prices']['prices'][$subComboGroupItem['subItemCode']]['PRECO']) ? $menu['prices']['prices'][$subComboGroupItem['subItemCode']]['PRECO'] : 0.01);
                        $subComboGroupItem['price'] = $subComboGroupItem['fullPrice'];
  
                        //Combo diff price
                        if(!empty($menu['prices']['comboDiffPrices'][$subComboGroupItem['subItemCode']][$subItemProductGroup['subProductGroupCode']][$comboGroupItem['subItemCode']])){
                          if($menu['prices']['comboDiffPrices'][$subComboGroupItem['subItemCode']][$subItemProductGroup['subProductGroupCode']][$comboGroupItem['subItemCode']]['IDDESCACRPROMO'] === 'D'){
                            if($menu['prices']['comboDiffPrices'][$subComboGroupItem['subItemCode']][$subItemProductGroup['subProductGroupCode']][$comboGroupItem['subItemCode']]['IDPERCVALOR'] === 'V'){
                              $subComboGroupItem['price'] -= $menu['prices']['comboDiffPrices'][$subComboGroupItem['subItemCode']][$subItemProductGroup['subProductGroupCode']][$comboGroupItem['subItemCode']]['VRDESCACR'];
                            } else{
                              $subComboGroupItem['price'] -= ($subComboGroupItem['price'] * (($menu['prices']['comboDiffPrices'][$subComboGroupItem['subItemCode']][$subItemProductGroup['subProductGroupCode']][$comboGroupItem['subItemCode']]['VRDESCACR']) / 100));
                            }
                          } else{
                            if($menu['prices']['comboDiffPrices'][$subComboGroupItem['subItemCode']][$subItemProductGroup['subProductGroupCode']][$comboGroupItem['subItemCode']]['IDPERCVALOR'] === 'V'){
                              $subComboGroupItem['price'] += $menu['prices']['comboDiffPrices'][$subComboGroupItem['subItemCode']][$subItemProductGroup['subProductGroupCode']][$comboGroupItem['subItemCode']]['VRDESCACR'];
                            } else{
                              $subComboGroupItem['price'] += ($subComboGroupItem['price'] * (($menu['prices']['comboDiffPrices'][$subComboGroupItem['subItemCode']][$subItemProductGroup['subProductGroupCode']][$comboGroupItem['subItemCode']]['VRDESCACR']) / 100));
                            }
                          }
                        }
  
                        //Daily price
                        if(!empty($menu['prices']['dailyPrices'][$subComboGroupItem['subItemCode']])){
                          if($menu['prices']['dailyPrices'][$subComboGroupItem['subItemCode']]['IDDESCACREPR'] === 'D'){
                            if($menu['prices']['dailyPrices'][$subComboGroupItem['subItemCode']]['IDPERVALORPR'] === 'V'){
                              $subComboGroupItem['price'] -= $menu['prices']['dailyPrices'][$subComboGroupItem['subItemCode']]['VRPRECODIA'];
                            } else{
                              $subComboGroupItem['price'] -= ($subComboGroupItem['price'] * ($menu['prices']['dailyPrices'][$subComboGroupItem['subItemCode']]['VRPRECODIA'] / 100));
                            }
                          } else{
                            if($menu['prices']['dailyPrices'][$subComboGroupItem['subItemCode']]['IDPERVALORPR'] === 'V'){
                              $subComboGroupItem['price'] += $menu['prices']['dailyPrices'][$subComboGroupItem['subItemCode']]['VRPRECODIA'];
                            } else{
                              $subComboGroupItem['price'] += ($subComboGroupItem['price'] * ($menu['prices']['dailyPrices'][$subComboGroupItem['subItemCode']]['VRPRECODIA'] / 100));
                            }
                          }
                        }
  
                        $subComboGroupItem['price'] = floatval(bcadd((string)$subComboGroupItem['price'], '0.00', 2));
                        $subComboGroupItem['discount'] = floatval(bcsub((string)$subComboGroupItem['fullPrice'], $subComboGroupItem['price'], 2));
                        if($subComboGroupItem['discount'] < 0.00){
                          $subComboGroupItem['addition'] = -1 * $subComboGroupItem['discount'];
                          $subComboGroupItem['discount'] = 0.0;
                        } else{
                          $subComboGroupItem['addition'] = 0.0;
                        }

                        if($subComboGroupItem['fullPrice'] === 0.0){
                          unset($subItemProductGroup['subProductGroupItems'][$subComboKey]);
                          continue;
                        }
  
                        //Combo item obs price
                        foreach ($subComboGroupItem['subItemObservationsChoice'] as &$sbCbItmObservationGroup){
                          foreach($sbCbItmObservationGroup['subItemObservations'] as &$sbCbItmObservationChoice){
                            if($sbCbItmObservationChoice['subItemObsType'] === 'A') {
                              $sbCbItmObservationChoice['fullPrice'] = (!empty($menu['prices']['prices'][$sbCbItmObservationChoice['subItemObsProductCode']]['PRECO']) ? $menu['prices']['prices'][$sbCbItmObservationChoice['subItemObsProductCode']]['PRECO'] : 0.01);
                              $sbCbItmObservationChoice['price'] = $sbCbItmObservationChoice['fullPrice'];
    
                              //Daily price
                              if(!empty($menu['prices']['dailyPrices'][$sbCbItmObservationChoice['subItemObsProductCode']])){
                                if($menu['prices']['dailyPrices'][$sbCbItmObservationChoice['subItemObsProductCode']]['IDDESCACREPR'] === 'D'){
                                  if($menu['prices']['dailyPrices'][$sbCbItmObservationChoice['subItemObsProductCode']]['IDPERVALORPR'] === 'V'){
                                    $sbCbItmObservationChoice['price'] -= $menu['prices']['dailyPrices'][$sbCbItmObservationChoice['subItemObsProductCode']]['VRPRECODIA'];
                                  } else{
                                    $sbCbItmObservationChoice['price'] -= ($sbCbItmObservationChoice['price'] * ($menu['prices']['dailyPrices'][$sbCbItmObservationChoice['subItemObsProductCode']]['VRPRECODIA'] / 100));
                                  }
                                } else{
                                  if($menu['prices']['dailyPrices'][$sbCbItmObservationChoice['subItemObsProductCode']]['IDPERVALORPR'] === 'V'){
                                    $sbCbItmObservationChoice['price'] += $menu['prices']['dailyPrices'][$sbCbItmObservationChoice['subItemObsProductCode']]['VRPRECODIA'];
                                  } else{
                                    $sbCbItmObservationChoice['price'] += ($sbCbItmObservationChoice['price'] * ($menu['prices']['dailyPrices'][$sbCbItmObservationChoice['subItemObsProductCode']]['VRPRECODIA'] / 100));
                                  }
                                }
                              }
            
                              $sbCbItmObservationChoice['price'] = floatval(bcadd((string)$sbCbItmObservationChoice['price'], '0.00', 2));
                              $sbCbItmObservationChoice['discount'] = floatval(bcsub((string)$sbCbItmObservationChoice['fullPrice'], $sbCbItmObservationChoice['price'], 2));
                              if($sbCbItmObservationChoice['discount'] < 0.00){
                                $sbCbItmObservationChoice['addition'] = -1 * $sbCbItmObservationChoice['discount'];
                                $sbCbItmObservationChoice['discount'] = 0.0;
                              } else{
                                $sbCbItmObservationChoice['addition'] = 0.0;
                              }
                            } else {
                              $sbCbItmObservationChoice['fullPrice'] = 0.0;
                              $sbCbItmObservationChoice['price'] = 0.0;
                              $sbCbItmObservationChoice['discount'] = 0.0;
                              $sbCbItmObservationChoice['addition'] = 0.0;
                            }
                          }
                        }
  
                        if(isset($sbCbItmObservationChoice)) unset($sbCbItmObservationChoice);
  
                      }
  
                      if(isset($subComboGroupItem)) unset($subComboGroupItem);
                    }
  
                    if(isset($subItemProductGroup)) unset($subItemProductGroup);
                  }
  
                  if(isset($comboGroupItem)) unset($comboGroupItem);
                }
  
                if(isset($productGroup)) unset($productGroup);
              }

              if(isset($product)) unset($product);
            }

            if(isset($subGroups)) unset($subGroups);
          }

          if(isset($group)) unset($group);

          unset($menu['combos']);
          unset($menu['prices']);

          $response->addDataSet(new DataSet('', $menu)); 
      } catch (\Exception $e) {
          $message = 'Erro ao buscar cardápio do caixa. '.$e->getMessage();
          $response->addMessage(new Message($message, self::ERROR_MESSAGE));
      }
  
  }

    public function V3_getPreselectedProds(Request $request, Response $response) {
      $inputJSON = file_get_contents('php://input');
      $input = json_decode($inputJSON, true);

      try{
        if (!isset($input)) {
          throw new \Exception('Não foram recebidos parâmetros');
        }

        $params = $input;
        $subsidiaryCode = isset($params['subsidiaryCode']) ? $params['subsidiaryCode'] : null;
        $promoProd = $params['promoProdCode'];
        $promoProdGroups = $params['promoProdGroups'];

        $data = array();

        foreach ($promoProdGroups as $comboGroup) {
          $branchPreSelectedProds = $this->menuService->getPreselectedProdsByBranch($subsidiaryCode, $promoProd, $comboGroup);
          $branchMandatoryProds = $this->menuService->getMandatoryProdsByBranch($subsidiaryCode, $promoProd, $comboGroup);
          $branchMainProducts = $this->menuService->getMainProductsByBranch($subsidiaryCode, $promoProd, $comboGroup);

          if(!empty($branchPreSelectedProds['data'])){
            $preSelectedProds = $branchPreSelectedProds;
            $mandatoryProds = $branchMandatoryProds;
            $mainProducts = $branchMainProducts;
          }else{
            $preSelectedProds = $this->menuService->getPreselectedProds($promoProd, $comboGroup);
            $mandatoryProds = $this->menuService->getMandatoryProds($promoProd, $comboGroup);
            $mainProducts = $this->menuService->getMainProducts($promoProd, $comboGroup);
          }

          if($preSelectedProds['error'] === false){
            if(empty($preSelectedProds['data'])){
              $data[$comboGroup]['preSelected'] = null;  
            } else{
              $data[$comboGroup]['preSelected'] = array();
              foreach($preSelectedProds['data'] as $preSelectedProd){
                array_push($data[$comboGroup]['preSelected'], $preSelectedProd['CDPRODUTO']);
              }
            }
          } else{
            throw new \Exception($preSelectedProds['message']);
          }
        
          if($mandatoryProds['error'] === false){
            if(empty($mandatoryProds['data'])){
              $data[$comboGroup]['mandatory'] = null;  
            } else{
              $data[$comboGroup]['mandatory'] = array();
              foreach($mandatoryProds['data'] as $mandatoryProd){
                array_push($data[$comboGroup]['mandatory'], $mandatoryProd['CDPRODUTO']);
              }
            }
          } else{
            throw new \Exception($mandatoryProds['message']);
          }
          
          if($mainProducts['error'] === false){
            if(empty($mainProducts['data'])){
              $data[$comboGroup]['main'] = null;  
            } else{
              $data[$comboGroup]['main'] = array();
              foreach($mainProducts['data'] as $mandatoryProd){
                array_push($data[$comboGroup]['main'], $mandatoryProd['CDPRODUTO']);
              }
            }
          } else{
            throw new \Exception($mainProducts['message']);
          }
        }

        $response->addDataSet(new DataSet('', $data)); 
      } catch (\Exception $e) {
        $message = 'Erro ao buscar produtos preselecionados, obrigatorios e principais. '.$e->getMessage();
        $response->addMessage(new Message($message, self::ERROR_MESSAGE));
      }
    }

    public function V3_getBloqProducts(Request $request, Response $response) {
        $inputJSON = file_get_contents('php://input');
        $input = json_decode($inputJSON, true);

        try {
            if (!isset($input)) {
                throw new \Exception('Não foram recebidos parâmetros');
            }

            $params = $input;
            $subsidiaryCode = $params['subsidiaryCode'];
            $storeCode      = $params['storeCode'];

            // verifica a existência da filial
            if (!$this->util->subsidiaryExists($subsidiaryCode)) {
                throw new \Exception('Filial inexistente.');
            }
            
            // verifica a existência da loja relacionada à filial
            if (!$this->util->storeExists($subsidiaryCode, $storeCode)) {
                throw new \Exception('Loja inexistente.');
            }

            $bloqProdsTAA = $this->menuService->getBloqProdsTAA($subsidiaryCode, $storeCode);
            $generalBloqProds = $this->menuService->getGeneralBloqProds($subsidiaryCode, $storeCode);

            $finalBloqProds = $this->menuService->buildBloqProds($bloqProdsTAA, $generalBloqProds);

            $response->addDataSet(new DataSet('', $finalBloqProds)); 
        } catch (\Exception $e) {
            $message = 'Erro ao buscar produtos bloqueados. '.$e->getMessage();
            $response->addMessage(new Message($message, self::ERROR_MESSAGE));
        }
    }

    public function getPosTefData(Request $request, Response $response){
        $inputJSON = file_get_contents('php://input');
        $input = json_decode($inputJSON, true);

        try {
            if (!isset($input)) {
                throw new \Exception('Não foram recebidos parâmetros');
            }
            
            $data   = array(); // dados a serem retornados

            $params = $input;
            $subsidiaryCode = $params['subsidiaryCode'];
            $storeCode      = $params['storeCode'];
            
            // verifica a existência da filial
            if (!$this->util->subsidiaryExists($subsidiaryCode)) {
                throw new \Exception('Filial inexistente.');
            }
            
            // verifica a existência da loja relacionada à filial
            if (!$this->util->storeExists($subsidiaryCode, $storeCode)) {
                throw new \Exception('Loja inexistente.');
            }
            
            $returnTefData = $this->menuService->getPosTefData($subsidiaryCode, $storeCode);
            $branchParams = $this->menuService->getBranchParams($subsidiaryCode);
            
            $posData = array();
            foreach($returnTefData['data'] as $tefPos) {
                $pos = array(
                    'storeCNPJ'       => $tefPos["NRCNPJLOJA"],
                    'ipSiTef'         => $tefPos["DSENDIPSITEF"],
                    'tefCode'         => $tefPos["CDTERTEF"],
                    'storeTefCode'    => $tefPos["CDLOJATEF"],
                    'posCode'         => $tefPos["CDCAIXA"],
                    'posName'         => $tefPos["NMCAIXA"],
                    'posType'         => $tefPos["IDHABCAIXAVENDA"],
                    'posCollector'    => $tefPos["IDCOLETOR"] == 'C' ? 'S':'N',
                    'sendingCupEmail' => $tefPos["IDENVCUPOMEMAIL"],
                    'usesOpeningLock' => isset($branchParams['IDBLOQMAISABERT']) ? $branchParams['IDBLOQMAISABERT'] : 'N',
                    'askPhoneNumber'  => isset($branchParams['IDSOLICITATEL']) ? $branchParams['IDSOLICITATEL'] : 'N',
                    'companyCodCRM'   => $tefPos["CDCODECRMBONUS"],
                    'authCRM'         => $tefPos["CDAUTHCRMBONUS"],
                    'apiUrlCRM'       => self::CRM_BONUS_URL,
                    'useDigitalPager' => !empty($branchParams['IDUSAPGEDIG']) ? $branchParams['IDUSAPGEDIG'] : 'N',
                    'apiUrlCashback'  => !empty($branchParams['CDURLFIDONLINE']) ? $branchParams['CDURLFIDONLINE'] : '',
                    'weighingConfig'  => null,
                    'useNFSe'         => !empty($branchParams['IDUTLNFSVENCONS']) ? $branchParams['IDUTLNFSVENCONS'] : 'N'
                );

                if(!empty($tefPos['IDLCDBARBALATOL']) && $tefPos['IDLCDBARBALATOL'] === 'S') {
                    $pos['weighingConfig'] = [
                        'initialBarProd' => $tefPos['NRPOSINICODBARR'],
                        'finalBarProd'   => $tefPos['NRPOSFINCODBARR'],
                    ];

                    if(!empty($tefPos['NRPOSINIQTCDBAR']) && !empty($tefPos['NRPOSFINQTCDBAR'])) {
                        $pos['weighingConfig']['initialQtty'] = $tefPos['NRPOSINIQTCDBAR'];
                        $pos['weighingConfig']['finalQtty']   = $tefPos['NRPOSFINQTCDBAR'];
                    }
                    else if (!empty($tefPos['NRPOSINIVRCDBAR']) && !empty($tefPos['NRPOSFINVRCDBAR'])) {
                        $pos['weighingConfig']['initialValue'] = $tefPos['NRPOSINIVRCDBAR'];
                        $pos['weighingConfig']['finalValue']   = $tefPos['NRPOSFINVRCDBAR'];
                    }
                    else {
                        throw new \Exception("Configuração de balança toledo Prix IV inválida para o caixa: {$tefPos["CDCAIXA"]} - {$tefPos["NMCAIXA"]}.");
                    }
                }
                $posData[] = $pos;
            }

            $response->addDataSet(new DataSet('', array('posTefData' => $posData)));

        } catch (\Exception $e) {
            $message = 'Erro ao buscar os dados do TEF dos caixas. '.$e->getMessage();
            $response->addMessage(new Message($message, self::ERROR_MESSAGE));
        }
    }

    public function getPayments(Request $request, Response $response) {
		  ini_set('memory_limit', '512M');
		
        try {
            $inputJSON = file_get_contents('php://input');
            $input = json_decode($inputJSON, true);

            if (isset($input['subsidiaryCode'])) 
                $cdfilial = $input['subsidiaryCode'];
            else 
                throw new \Exception('Filial não informada.');

            if (isset($input['posCode'])) $cdcaixa = $input['posCode'];
            else throw new \Exception('Caixa não informado.');
            
            $payments = $this->menuService->getPayments();
            if($payments['error']) throw new \Exception($payments['message']);
            
            $paymentsLayout = $this->menuService->getPaymentsLayout($cdfilial, $cdcaixa);
            if($paymentsLayout['error']) throw new \Exception($paymentsLayout['message']);

            $response->addDataSet(
                new DataSet(
                    '',
                    array(
                        'payments'     => $payments['data'],
                        'layout'       => $paymentsLayout['data']
                    )
                )
            );
            
        } catch (\Exception $e) {
            $message = 'Erro ao buscar os tipos de recebimentos. ' . $e->getMessage();
            $response->addMessage(new Message($message, self::ERROR_MESSAGE));
        }
    }
    public function getProductSuggestion(Request $request, Response $response) {
		    ini_set('memory_limit', '512M');
        try {
            $inputJSON = file_get_contents('php://input');
            $input = json_decode($inputJSON, true);

            if (isset($input['subsidiaryCode'])) 
                $cdfilial = $input['subsidiaryCode'];
            else 
                throw new \Exception('Filial não informada.');
            
            if (isset($input['posCode'])) 
                $cdcaixa = $input['posCode'];
            else 
                throw new \Exception('Caixa não informado.');

            if (isset($input['products'])) 
                $products = $input['products'];
            else 
                throw new \Exception('Produto(s) não informado(s).');

            $suggestions = $this->menuService->getProductSuggestion($cdfilial, $cdcaixa, $products);

            if($suggestions['error']) throw new \Exception($suggestions['message']);

            $suggestionsData = array();
            foreach($suggestions['data'] as $suggestion){
                $suggestionData = array(
                    'suggestionId' => $suggestion['IDSUGESTAOVENDA'],
                    'productCode' => $suggestion['CDPRODUTO'],
                    'productSuggestion' => $suggestion['CDSUGESTAOITEM'],
                    'suggestionType' => $suggestion['IDSUGESTAOTIPO']
                );
                if(!isset($suggestionsData[$suggestion['CDPRODUTO']])) $suggestionsData[$suggestion['CDPRODUTO']] = array($suggestionData);
                else $suggestionsData[$suggestion['CDPRODUTO']][] = $suggestionData;
            }

            $response->addDataSet(
                new DataSet(
                    '',
                    array(
                        'suggestions'     => $suggestionsData
                    )
                )
            );
        } catch (\Exception $e) {
            $message = 'Erro ao buscar sugestão de venda. ' . $e->getMessage();
            $response->addMessage(new Message($message, self::ERROR_MESSAGE));
        }
    }

    public function addBloqProducts(Request $request, Response $response) {
		    ini_set('memory_limit', '512M');
		    $this->util->beginTransaction();
        try {
            $inputJSON = file_get_contents('php://input');
            $input = json_decode($inputJSON, true);

            if (isset($input['subsidiaryCode'])) 
                $cdfilial = $input['subsidiaryCode'];
            else 
                throw new \Exception('Filial não informada.');
            
            if (isset($input['storeCode'])) 
                $cdloja = $input['storeCode'];
            else 
                throw new \Exception('Loja não informada.');

            if (isset($input['products'])) 
                $produtos = $input['products'];
            else 
                throw new \Exception('Produtos não informados.');
        
            $sellerReturn = $this->orderService->getStandardSeller($cdfilial)['data'];
            $cdoperador = $sellerReturn['CDOPERADOR'];
            $nrorg = $sellerReturn['NRORG'];

            $str = "";
            foreach($produtos as $prod){
                $this->menuService->insertBloqProd($cdfilial, $cdloja, $cdoperador, $prod, $nrorg);
                $str .= $prod.", ";
            }

            $this->util->commit();
            $response->addMessage(new Message("Os produtos $str foram bloqueados.", self::GENERAL_MESSAGE));
        } catch (\Exception $e) {
            $this->util->rollBack();
            $this->util->saveLog($e->getMessage(),'Error','BLOQ-PRODS-');
            $message = 'Erro ao bloquear os produtos. ' . $e->getMessage();
            $response->addMessage(new Message($message, self::ERROR_MESSAGE));
        }
    }

    public function removeBloqProducts(Request $request, Response $response) {
      ini_set('memory_limit', '512M');
      $this->util->beginTransaction();
      try {
          $inputJSON = file_get_contents('php://input');
          $input = json_decode($inputJSON, true);

          if (isset($input['subsidiaryCode'])) 
              $cdfilial = $input['subsidiaryCode'];
          else 
              throw new \Exception('Filial não informada.');
          
          if (isset($input['storeCode'])) 
              $cdloja = $input['storeCode'];
          else 
              throw new \Exception('Loja não informada.');

          if (isset($input['products'])) 
              $produtos = $input['products'];
          else 
              throw new \Exception('Produtos não informados.');
      
          // $sellerReturn = $this->orderService->getStandardSeller($cdfilial)['data'];
          // $cdoperador = $sellerReturn['CDOPERADOR'];
          // $nrorg = $sellerReturn['NRORG'];

          $str = "";
          foreach($produtos as $prod){
              $this->menuService->deleteBloqProdTAA($cdfilial, $cdloja, $prod);
              $this->menuService->deleteBloqProdVND($cdfilial, $cdloja, $prod);
              $str .= $prod.", ";
          }

          $this->util->commit();
          $response->addMessage(new Message("Os produtos $str foram desbloqueados.", self::GENERAL_MESSAGE));
      } catch (\Exception $e) {
          $this->util->rollBack();
          $this->util->saveLog($e->getMessage(),'Error','BLOQ-PRODS-');
          $message = 'Erro ao desbloquear os produtos. ' . $e->getMessage();
          $response->addMessage(new Message($message, self::ERROR_MESSAGE));
      }
  }

  public function getWeightProducts(Request $request, Response $response) {
    ini_set('zlib.output_compression_level', 1);
    ob_start('ob_gzhandler');

    $inputJSON = file_get_contents('php://input');
    $input = json_decode($inputJSON, true);

    try {
        if (!isset($input)) {
            throw new \Exception('Não foram recebidos parâmetros');
        }
        
        $data   = array(); // dados a serem retornados

        $params = $input;
        $subsidiaryCode = $params['subsidiaryCode'];
        $storeCode      = $params['storeCode'];
        $posCode        = $params['posCode'];
        $consumerType   = $params['consumerType'];
        $maskedHour     = $params['hour'];
        $hour           = str_replace(':','',$maskedHour);

        if(!empty($params['clientCode'])){
            $clientCode = $params['clientCode'];
        } else{
            $clientCode = null;
        }
        
        // verifica a existência da filial
        if (!$this->util->subsidiaryExists($subsidiaryCode)) {
            throw new \Exception('Filial inexistente.');
        }
        
        // verifica a existência da loja relacionada à filial
        if (!$this->util->storeExists($subsidiaryCode, $storeCode)) {
            throw new \Exception('Loja inexistente.');
        }
        
        // verifica a existência do caixa relacionado à loja
        if (!$this->util->posExists($subsidiaryCode, $storeCode, $posCode)) {
            throw new \Exception('Caixa inexistente.');
        }
        
        $confTelaCaixa = $this->menuService->getConfTelaCaixaEx($subsidiaryCode, $posCode);

        if($confTelaCaixa['error']) {
            $message = $confTelaCaixa['message'];
            $response->addMessage(new Message($message, self::ERROR_MESSAGE));
        }

        $nrConfTela = $confTelaCaixa['data']['NRCONFTELA'];
        $products = $this->menuService->getWeightProducts($nrConfTela, $subsidiaryCode, $storeCode, $clientCode, $consumerType);

        $response->addDataSet(
          new DataSet(
              '',
              array(
                  $products['data']
              )
            )
        );
    } catch (\Exception $e) {
        $message = 'Erro ao montar layout de produtos fracionados. ' . $e->getMessage();
        $response->addMessage(new Message($message, self::ERROR_MESSAGE));
    }
  }

  public function getDifferentiatedPrice(Request $request, Response $response) {
    ini_set('zlib.output_compression_level', 1);
    ob_start('ob_gzhandler');

    $inputJSON = file_get_contents('php://input');
    $input = json_decode($inputJSON, true);

    try {
        if (!isset($input)) {
            throw new \Exception('Não foram recebidos parâmetros');
        }
        
        $data   = array(); // dados a serem retornados

        $params = $input;
        $subsidiaryCode = $params['subsidiaryCode'];
        $storeCode      = $params['storeCode'];
        $consumerCode   = $params['consumerCode'];
        $clientCode     = $params['clientCode'];

        if(empty($params['clientCode'])){
          throw new \Exception('Código de Cliente não informado.');
        }

        if (empty($params['consumerCode'])){
          throw new \Exception('Código de consumidor não informado.');
        }
        
        // verifica a existência da filial
        if (!$this->util->subsidiaryExists($subsidiaryCode)) {
            throw new \Exception('Filial inexistente.');
        }
        
        // verifica a existência da loja relacionada à filial
        if (!$this->util->storeExists($subsidiaryCode, $storeCode)) {
            throw new \Exception('Loja inexistente.');
        }
        
        $consumerType = $this->menuService->getconsumerType($clientCode, $consumerCode);

        $priceProducts = $this->menuService->getDifferentiatedPrice($subsidiaryCode, $storeCode, $clientCode, $consumerCode, $consumerType['data']);
        if (isset($branchPrices['error']) && isset($branchPrices['error']))
                throw new \Exception('');
        $response->addDataSet(
          new DataSet(
              '',
              $priceProducts
            )
        );
    } catch (\Exception $e) {
        $message = 'Erro ao montar layout de consulta de preço. ' . $e->getMessage();
        $response->addMessage(new Message($message, self::ERROR_MESSAGE));
    }
  }

  public function getProdAvailabilityPerHour(Request $request, Response $response) {

    $inputJSON = file_get_contents('php://input');
    $input = json_decode($inputJSON, true);

    try {
        if (!isset($input)) {
            throw new \Exception('Não foram recebidos parâmetros');
        }
        
        $data   = array(); // dados a serem retornados

        $params = $input;
        $subsidiaryCode = $params['subsidiaryCode'];
        $storeCode      = $params['storeCode'];
        $consumerCode   = isset($params['consumerCode']) ? $params['consumerCode'] : null;
        $clientCode     = isset($params['clientCode']) ? $params['clientCode'] : null;


        if(empty($subsidiaryCode))
          throw new \Exception('Filial deve ser informada.');

        if(empty($storeCode))
          throw new \Exception('Loja deve ser informada.');

        
        // verifica a existência da filial
        if (!$this->util->subsidiaryExists($subsidiaryCode)) {
            throw new \Exception('Filial inexistente.');
        }
        
        // verifica a existência da loja relacionada à filial
        if (!$this->util->storeExists($subsidiaryCode, $storeCode)) {
            throw new \Exception('Loja inexistente.');
        }

        $clientCode = empty($clientCode) ? $this->util->getDefaultClient($subsidiaryCode) : $clientCode;
        $unavailableProducts = $this->menuService->getUnavailableProductsPerHour($subsidiaryCode, $storeCode, $clientCode);

        $response->addDataSet(
          new DataSet('', $unavailableProducts));
          
    } catch (\Exception $e) {
        $message = 'Erro na consulta de produtos indisponiveis. ' . $e->getMessage();
        $response->addMessage(new Message($message, self::ERROR_MESSAGE));
    }
  }

  public function getCustomTaaConfigurations(Request $request, Response $response){
    try{
      $customConfigs = $this->menuService->getCustomTaaConfigurations();

      $message = '';

      if(empty($customConfigs['CORESJSON'])){
        $message .= "Parametrizacao das informacoes do campo 'coresInfo' inxesistente.";
      }

      if(empty($customConfigs['JSONB64CUSTOMMEDIA'])){
        if(!empty($message)){
          $message .= ' ';
        }
        $message .= "Parametrizacao das informacoes do campo 'customMediaInfo' inxesistente.";
      }
      
      $taaCustomConfigs = [
        'coresInfo' => !empty($customConfigs['CORESJSON']) ? json_decode($customConfigs['CORESJSON'], true) : [],
        'customMediaInfo' => !empty($customConfigs['JSONB64CUSTOMMEDIA']) ? json_decode($customConfigs['JSONB64CUSTOMMEDIA'], true) : []
      ];

      $response->addMessage(new Message($message, self::GENERAL_MESSAGE));

      $response->addDataSet(
        new DataSet('', $taaCustomConfigs)
      );
    } catch (\Exception $e) {
        $message = 'Erro na consulta de configuracoes pesonalizadas do TAA. ' . $e->getMessage();
        $response->addMessage(new Message($message, self::ERROR_MESSAGE));
    }
  }

  public function offers(Request $request, Response $response){
    $inputJSON = file_get_contents('php://input');
    $input = json_decode($inputJSON, true);

    try {
        if (!isset($input)) {
            throw new \Exception('Não foram recebidos parâmetros');
        }

        $params = $input;
        $subsidiaryCode = $params['subsidiaryCode'];

        if(empty($subsidiaryCode))
            throw new \Exception('Filial deve ser informada.');

        $offers = $this->menuService->getOffers($subsidiaryCode);

        $response->addDataSet(
            new DataSet('', $offers)
        );
    } catch (\Exception $e) {
        $message = 'Erro na consulta de campanhas promocionais. ' . $e->getMessage();
        $response->addMessage(new Message($message, self::ERROR_MESSAGE));
    }
  }

  public function offersbyCombinations(Request $request, Response $response) {
    $inputJSON = file_get_contents('php://input');
    $input = json_decode($inputJSON, true);

    try {
        if (!isset($input)) {
            throw new \Exception('Não foram recebidos parâmetros');
        }

        $cart = $input;
        $subsidiaryCode = $cart['subsidiaryCode'];
      
        if(empty($subsidiaryCode))
            throw new \Exception('Filial deve ser informada.');
      
        if(empty($cart['selectedProducts']))
            throw new \Exception('selectedProducts não pode ser vazio.');
      
        $clientDefault = $this->util->getDefaultClient($subsidiaryCode);
        $clientCode    = !empty($cart['clientCode']) ? $cart['clientCode'] : $clientDefault['CDCLIENTE'];
        $consumerType  = !empty($cart['consumerType']) ? $cart['consumerType'] : $clientDefault['CDTIPOCONS'];
      
        $validOffers = $this->menuService->getValidOffers($subsidiaryCode, $clientCode, $consumerType, $cart);

        if(empty($validOffers)) {
            return $response->addDataSet(new DataSet('', $cart));
        }

        $bestOffer = 0;
        $bestValue = 0;

        foreach($validOffers as $key => $offer) {
            $VRDESCACRE = $this->util->removeDecimal($offer['VRDESCACRE'], 3);
        
            if($VRDESCACRE > $bestValue) {
                $bestValue = $VRDESCACRE;
                $bestOffer = $key;
            }
        }

        $type =  $validOffers[$bestOffer]['IDDESCACRE'];
        $applyIn = $validOffers[$bestOffer]['IDAPLICADESACR'];
        $priceModifierType = $validOffers[$bestOffer]['IDPERCVALOR'];
      
        $fatherDiscount = 0;
        $fatherAddition = 0;

        $qtdItens = 0;
        $productsValue = 0;
        $divisionByItems = [];

        foreach ($cart['selectedProducts'] as $key => &$product) {
            if($cart['productsPrint'] == 'S') {
                if($product['groupPrint'] == 'N' && empty($product['childItems'])) {
                    if($type == 'D') {
                        if($priceModifierType == 'P') {
                            if ($applyIn == '4') {
                                $discount = floatval(bcdiv((bcmul($product['fullPrice'], $bestValue, 2)), 100 , 2));
                                $fatherDiscount = bcadd($fatherDiscount, $discount, 2);
                            } else {
                                if($product['productCode'] == $validOffers[$bestOffer]['CDPRODPRIN'] && $applyIn == '1') {
                                    $discount = floatval(bcdiv((bcmul($product['fullPrice'], $bestValue, 2)), 100 , 2));
                                    $fatherDiscount = bcadd($fatherDiscount, $discount, 2);
                                } else if($product['productCode'] == $validOffers[$bestOffer]['CDPRODCOMB'] && $applyIn == '2') {
                                    $discount = floatval(bcdiv((bcmul($product['fullPrice'], $bestValue, 2)), 100 , 2));
                                    $fatherDiscount = bcadd($fatherDiscount, $discount, 2);
                                } else if($product['productCode'] == $validOffers[$bestOffer]['CDPRODCOMB2'] && $applyIn == '3') {
                                    $discount = floatval(bcdiv((bcmul($product['fullPrice'], $bestValue, 2)), 100 , 2));
                                    $fatherDiscount = bcadd($fatherDiscount, $discount, 2);
                                } else if($product['productCode'] == $validOffers[$bestOffer]['CDPRODCOMB4'] && $applyIn == '5') {
                                    $discount = floatval(bcdiv((bcmul($product['fullPrice'], $bestValue, 2)), 100 , 2));
                                    $fatherDiscount = bcadd($fatherDiscount, $discount, 2);
                                }
                            }
                        } else {
                            if($fatherDiscount == 0) { 
                                if($applyIn == '4') {
                                    $fatherDiscount = bcadd($fatherDiscount, $bestValue, 2);
                                } else {
                                    if($product['productCode'] == $validOffers[$bestOffer]['CDPRODPRIN'] && $applyIn == '1') {
                                        $discount = $bestValue;
                                        $fatherDiscount = bcadd($fatherDiscount, $discount, 2);
                                    } else if($product['productCode'] == $validOffers[$bestOffer]['CDPRODCOMB'] && $applyIn == '2') {
                                        $discount = $bestValue;
                                        $fatherDiscount = bcadd($fatherDiscount, $discount, 2);
                                    } else if($product['productCode'] == $validOffers[$bestOffer]['CDPRODCOMB2'] && $applyIn == '3') {
                                        $discount = $bestValue;
                                        $fatherDiscount = bcadd($fatherDiscount, $discount, 2);
                                    } else if($product['productCode'] == $validOffers[$bestOffer]['CDPRODCOMB4'] && $applyIn == '5') {
                                        $discount = $bestValue;
                                        $fatherDiscount = bcadd($fatherDiscount, $discount, 2);
                                    }
                                }
                            }
                        }
                    } 
                    //else {
                    //    if($priceModifierType == 'P') {
                    //        if ($applyIn == '4') {
                    //            $addition = floatval(bcdiv((bcmul($product['fullPrice'], $bestValue, 2)), 100 , 2));
                    //            $fatherAddition = bcadd($fatherAddition, $addition, 2);
                    //        } else {
                    //          if($product['productCode'] == $validOffers[$bestOffer]['CDPRODPRIN'] && $applyIn == '1') {
                    //              $addition = floatval(bcdiv((bcmul($product['fullPrice'], $bestValue, 2)), 100 , 2));
                    //              $fatherAddition = bcadd($fatherAddition, $addition, 2);
                    //          } else if($product['productCode'] == $validOffers[$bestOffer]['CDPRODCOMB'] && $applyIn == '2') {
                    //              $addition = floatval(bcdiv((bcmul($product['fullPrice'], $bestValue, 2)), 100 , 2));
                    //              $fatherAddition = bcadd($fatherAddition, $addition, 2);
                    //          } else if($product['productCode'] == $validOffers[$bestOffer]['CDPRODCOMB2'] && $applyIn == '3') {
                    //              $addition = floatval(bcdiv((bcmul($product['fullPrice'], $bestValue, 2)), 100 , 2));
                    //              $fatherAddition = bcadd($fatherAddition, $addition, 2);
                    //          } else if($product['productCode'] == $validOffers[$bestOffer]['CDPRODCOMB4'] && $applyIn == '5') {
                    //              $addition = floatval(bcdiv((bcmul($product['fullPrice'], $bestValue, 2)), 100 , 2));
                    //              $fatherAddition = bcadd($fatherAddition, $addition, 2);
                    //          }
                    //        }
                    //    } else {
                    //        if ($fatherAddition == 0) { 
                    //            $fatherAddition = bcadd($fatherAddition, $bestValue, 2);
                    //        }
                    //    }
                    //}
                } else if ($product['groupPrint'] == 'S' && empty($product['childItems'])) {
                    if($type == 'D') {
                        if($priceModifierType == 'P') {
                            $discount = floatval(bcdiv((bcmul($product['fullPrice'], $bestValue, 2)), 100 , 2));
                            if ($applyIn == '4') {
                                $product['discount'] = $discount;
                                $product['price']    = floatval(bcsub($product['fullPrice'], $discount, 2));

                            }

                            if($product['productCode'] == $validOffers[$bestOffer]['CDPRODPRIN'] && $applyIn == '1') {
                                $product['discount'] = $discount;
                                $product['price']    = bcsub($product['price'], $discount, 2);

                            } else if($product['productCode'] == $validOffers[$bestOffer]['CDPRODCOMB'] && $applyIn == '2') {
                                $product['discount'] = $discount;
                                $product['price']    = bcsub($product['price'], $discount, 2);

                            } else if($product['productCode'] == $validOffers[$bestOffer]['CDPRODCOMB2'] && $applyIn == '3') {
                                $product['discount'] = $discount;
                                $product['price']    = bcsub($product['price'], $discount, 2);

                            } else if($product['productCode'] == $validOffers[$bestOffer]['CDPRODCOMB4'] && $applyIn == '5') {
                                $product['discount'] = $discount;
                                $product['price']    = bcsub($product['price'], $discount, 2);
                            }

                            $product['discount'] = floatval($product['discount']);
                            $product['price']    = floatval($product['price']);
                        } else {
                            if ($applyIn == '4') {
                                $qtdItens += 1;
                                $productsValue = floatval(bcadd($productsValue, $product['fullPrice'], 2));
                                array_push($divisionByItems,
                                    [
                                        'index'       => $key,
                                        'productCode' => $product['productCode'],
                                        'childIndex'  => null,
                                        'price'       => $product['fullPrice'],
                                        'discount'    => 0,
                                        'addition'    => 0
                                    ]
                                );
                            } else {
                                if($product['productCode'] == $validOffers[$bestOffer]['CDPRODPRIN'] && $applyIn == '1') {
                                    $product['discount'] = floatval($bestValue);
                                    $product['price'] = floatval(bcsub($product['fullPrice'], $bestValue, 2));
                                } else if($product['productCode'] == $validOffers[$bestOffer]['CDPRODCOMB'] && $applyIn == '2') {
                                    $product['discount'] = floatval($bestValue);
                                    $product['price'] = floatval(bcsub($product['fullPrice'], $bestValue, 2));
                                } else if($product['productCode'] == $validOffers[$bestOffer]['CDPRODCOMB2'] && $applyIn == '3') {
                                    $product['discount'] = floatval($bestValue);
                                    $product['price'] = floatval(bcsub($product['fullPrice'], $bestValue, 2));
                                } else if($product['productCode'] == $validOffers[$bestOffer]['CDPRODCOMB4'] && $applyIn == '5') {
                                    $product['discount'] = floatval($bestValue);
                                    $product['price'] = floatval(bcsub($product['fullPrice'], $bestValue, 2));
                                }  
                            }
                        }

                        if($product['price'] < 0.01) {
                            $product['discount'] = floatval($product['fullPrice'] - 0.01);
                            $product['price']    = 0.01;
                        }
                    } else {
                        if($priceModifierType == 'P') {
                            $addition = floatval(bcdiv((bcmul($product['fullPrice'], $bestValue, 2)), 100 , 2));

                            if ($applyIn == '4') {
                                $product['price'] = floatval(bcadd($product['fullPrice'], $addition, 2));
                                $product['addition'] = $addition;
                            } else {
                                if($product['productCode'] == $validOffers[$bestOffer]['CDPRODPRIN'] && $applyIn == '1') {
                                    $product['addition'] = floatval($addition);
                                    $product['price'] = floatval(bcadd($product['fullPrice'], $addition, 2));
                                } else if($product['productCode'] == $validOffers[$bestOffer]['CDPRODCOMB'] && $applyIn == '2') {
                                    $product['addition'] = floatval($addition);
                                    $product['price'] = floatval(bcadd($product['fullPrice'], $addition, 2));
                                } else if($product['productCode'] == $validOffers[$bestOffer]['CDPRODCOMB2'] && $applyIn == '3') {
                                    $product['addition'] = floatval($addition);
                                    $product['price'] = floatval(bcadd($product['fullPrice'], $addition, 2));
                                } else if($product['productCode'] == $validOffers[$bestOffer]['CDPRODCOMB4'] && $applyIn == '5') {
                                    $product['addition'] = floatval($addition);
                                    $product['price'] = floatval(bcadd($product['fullPrice'], $addition, 2));
                                }
                            }
                        } else {
                            if ($applyIn == '4') {
                                $qtdItens += 1;
                                $productsValue = floatval(bcadd($productsValue, $product['fullPrice'], 2));
                                array_push($divisionByItems,
                                    [
                                        'index'       => $key,
                                        'productCode' => $product['productCode'],
                                        'childIndex'  => null,
                                        'price'       => $product['fullPrice'],
                                        'discount'    => 0,
                                        'addition'    => 0
                                    ]
                                );
                            } else {
                                if($product['productCode'] == $validOffers[$bestOffer]['CDPRODPRIN'] && $applyIn == '1') {
                                    $product['addition'] = floatval($bestValue);
                                    $product['price'] = floatval(bcadd($product['fullPrice'], $bestValue, 2));
                                } else if($product['productCode'] == $validOffers[$bestOffer]['CDPRODCOMB'] && $applyIn == '2') {
                                    $product['addition'] = floatval($bestValue);
                                    $product['price'] = floatval(bcadd($product['fullPrice'], $bestValue, 2));
                                } else if($product['productCode'] == $validOffers[$bestOffer]['CDPRODCOMB2'] && $applyIn == '3') {
                                    $product['addition'] = floatval($bestValue);
                                    $product['price'] = floatval(bcadd($product['fullPrice'], $bestValue, 2));
                                } else if($product['productCode'] == $validOffers[$bestOffer]['CDPRODCOMB4'] && $applyIn == '5') {
                                    $product['addition'] = floatval($bestValue);
                                    $product['price'] = floatval(bcadd($product['fullPrice'], $bestValue, 2));
                                }  
                            }
                        }
                    }
                } else if (!empty($product['childItems'])) {
                  foreach ($product['childItems'] as $childItems) {
                      if($type == 'D') {
                          if($priceModifierType == 'P') {
                              $discount = bcdiv((bcmul($childItems['fullPrice'], $bestValue, 2)), 100 , 2);
                              $fatherDiscount = bcadd($fatherDiscount, $discount, 2);
                          } else {
                              if($childItems['productCode'] == $validOffers[$bestOffer]['CDPRODPRIN'] && $applyIn == '1') {
                                $childItems['discount'] = $bestValue;
                                $childItems['price']    = bcsub($childItems['price'], $bestValue, 2);

                            } else if($childItems['productCode'] == $validOffers[$bestOffer]['CDPRODCOMB'] && $applyIn == '2') {
                                $childItems['discount'] = $bestValue;
                                $childItems['price']    = bcsub($childItems['price'], $bestValue, 2);

                            } else if($childItems['productCode'] == $validOffers[$bestOffer]['CDPRODCOMB2'] && $applyIn == '3') {
                                $childItems['discount'] = $bestValue;
                                $childItems['price']    = bcsub($childItems['price'], $bestValue, 2);

                            } else if($childItems['productCode'] == $validOffers[$bestOffer]['CDPRODCOMB4'] && $applyIn == '5') {
                                $childItems['discount'] = $bestValue;
                                $childItems['price']    = bcsub($childItems['price'], $bestValue, 2);
                            }
                          }
                      }
                      //else {
                      //    if($priceModifierType == 'P') {
                      //        $addition = bcdiv((bcmul($childItems['fullPrice'], $bestValue, 2)), 100 , 2);
                      //        $fatherAddition = bcadd($fatherAddition, $addition, 2);
                      //    } else {
                      //        $fatherAddition = bcadd($fatherAddition, $bestValue, 2);
                      //    }
                      //}
                  }
                } 
            } else {
                if(empty($product['childItems'])) {
                    if($type == 'D') {
                        if($priceModifierType == 'P') {
                            $discount = floatval(bcdiv((bcmul($product['fullPrice'], $bestValue, 2)), 100 , 2));

                            if ($applyIn == '4') {
                                $product['discount'] = $discount;
                                $product['price'] = floatval(bcsub($product['fullPrice'], $discount, 2));
                            }
                            
                        } else {
                            if ($applyIn == '4') {
                                $qtdItens += 1;
                                $productsValue = floatval(bcadd($productsValue, $product['fullPrice'], 2));
                                array_push($divisionByItems,
                                    [
                                        'index'       => $key,
                                        'productCode' => $product['productCode'],
                                        'childIndex'  => null,
                                        'price'       => $product['fullPrice'],
                                        'discount'    => 0,
                                        'addition'    => 0
                                    ]
                                );
                                
                            } if($product['productCode'] == $validOffers[$bestOffer]['CDPRODPRIN'] && $applyIn == '1') {
                                $product['discount'] = floatval($bestValue);
                                $product['price'] = floatval(bcsub($product['fullPrice'], $bestValue, 2));
                            } else if($product['productCode'] == $validOffers[$bestOffer]['CDPRODCOMB'] && $applyIn == '2') {
                                $product['discount'] = floatval($bestValue);
                                $product['price'] = floatval(bcsub($product['fullPrice'], $bestValue, 2));
                            } else if($product['productCode'] == $validOffers[$bestOffer]['CDPRODCOMB2'] && $applyIn == '3') {
                                $product['discount'] = floatval($bestValue);
                                $product['price'] = floatval(bcsub($product['fullPrice'], $bestValue, 2));
                            } else if($product['productCode'] == $validOffers[$bestOffer]['CDPRODCOMB4'] && $applyIn == '5') {
                                $product['discount'] = floatval($bestValue);
                                $product['price'] = floatval(bcsub($product['fullPrice'], $bestValue, 2));
                            }
                        }

                        if($product['price'] < 0.01) {
                            $product['discount'] = floatval($product['fullPrice'] - 0.01);
                            $product['price'] = 0.01;
                        }
                    } else {
                        if($priceModifierType == 'P') {
                            $addition = floatval(bcdiv((bcmul($product['fullPrice'], $bestValue, 2)), 100 , 2));

                            if ($applyIn == '4') {
                                $product['addition'] = $addition;
                                $product['price'] = floatval(bcadd($product['fullPrice'], $addition, 2));
                            }
                        } else {
                            if ($applyIn == '4') {
                                $qtdItens += 1;
                                array_push($divisionByItems,
                                    [
                                        'index'       => $key,
                                        'productCode' => $product['productCode'],
                                        'childIndex'  => null,
                                        'price'       => $product['fullPrice'],
                                        'discount'    => 0,
                                        'addition'    => 0
                                    ]
                                );
                            } else {
                              if($product['productCode'] == $validOffers[$bestOffer]['CDPRODPRIN'] && $applyIn == '1') {
                                  $product['discount'] = floatval($bestValue);
                                  $product['price'] = floatval(bcsub($product['fullPrice'], $bestValue, 2));
                              } else if($product['productCode'] == $validOffers[$bestOffer]['CDPRODCOMB'] && $applyIn == '2') {
                                  $product['discount'] = floatval($bestValue);
                                  $product['price'] = floatval(bcsub($product['fullPrice'], $bestValue, 2));
                              } else if($product['productCode'] == $validOffers[$bestOffer]['CDPRODCOMB2'] && $applyIn == '3') {
                                  $product['discount'] = floatval($bestValue);
                                  $product['price'] = floatval(bcsub($product['fullPrice'], $bestValue, 2));
                              } else if($product['productCode'] == $validOffers[$bestOffer]['CDPRODCOMB4'] && $applyIn == '5') {
                                  $product['discount'] = floatval($bestValue);
                                  $product['price'] = floatval(bcsub($product['fullPrice'], $bestValue, 2));
                              }
                          }
                        }
                    }
                } else {
                    foreach ($product['childItems'] as $childKey => &$childItems) {
                        if($type == 'D') {
                            if($priceModifierType == 'P') {
                                $discount = bcdiv((bcmul($childItems['fullPrice'], $bestValue, 2)), 100 , 2);
                                if ($applyIn == '4') {
                                    $childItems['discount'] = $discount;
                                    $childItems['price'] = floatval(bcsub($childItems['fullPrice'], $discount, 2));
                                }
                            } else {
                                if ($applyIn == '4') {
                                    $qtdItens += 1;
                                    $productsValue = floatval(bcadd($productsValue, $childItems['fullPrice'], 2));
                                    array_push($divisionByItems,
                                        [
                                            'index'       => $key,
                                            'productCode' => $childItems['productCode'],
                                            'childIndex'  => $childKey,
                                            'price'       => $childItems['fullPrice'],
                                            'discount'    => 0,
                                            'addition'    => 0
                                        ]
                                    );
                                } else {
                                    if($childItems['productCode'] == $validOffers[$bestOffer]['CDPRODPRIN'] && $applyIn == '1') {
                                        $childItems['discount'] = floatval($bestValue);
                                        $childItems['price'] = floatval(bcsub($childItems['fullPrice'], $bestValue, 2));
                                    } else if($childItems['productCode'] == $validOffers[$bestOffer]['CDPRODCOMB'] && $applyIn == '2') {
                                        $childItems['discount'] = floatval($bestValue);
                                        $childItems['price'] = floatval(bcsub($childItems['fullPrice'], $bestValue, 2));
                                    } else if($childItems['productCode'] == $validOffers[$bestOffer]['CDPRODCOMB2'] && $applyIn == '3') {
                                        $childItems['discount'] = floatval($bestValue);
                                        $childItems['price'] = floatval(bcsub($childItems['fullPrice'], $bestValue, 2));
                                    } else if($childItems['productCode'] == $validOffers[$bestOffer]['CDPRODCOMB4'] && $applyIn == '5') {
                                        $childItems['discount'] = floatval($bestValue);
                                        $childItems['price'] = floatval(bcsub($childItems['fullPrice'], $bestValue, 2));
                                    }
                                }
                            }

                            if($childItems['price'] < 0.01) {
                                $childItems['discount'] = floatval($childItems['fullPrice'] - 0.01);
                                $childItems['price'] = 0.01;
                            }
                        } else {
                            if($priceModifierType == 'P') {
                                $addition = bcdiv((bcmul($childItems['fullPrice'], $bestValue, 2)), 100 , 2);
                                
                                if ($applyIn == '4') {
                                    $childItems['addition'] = $addition;
                                    $childItems['price'] = floatval(bcadd($childItems['fullPrice'], $addition, 2));
                                }
                            } else {
                                if ($applyIn == '4') {
                                    $qtdItens += 1;
                                    array_push($divisionByItems,
                                        [
                                            'index'       => $key,
                                            'productCode' => $childItems['productCode'],
                                            'childIndex'  => $childKey,
                                            'price'       => $childItems['fullPrice'],
                                            'discount'    => 0,
                                            'addition'    => 0
                                        ]
                                    );
                                }
                            }
                        }
                    }
                    if($childItems) {
                        unset($childItems);
                    }
                }
            }
        }

        if(isset($product))
            unset($product);
        
        if($type == 'A' && $qtdItens > 0) {
            $dividedValue = floatval(bcdiv($bestValue, $qtdItens, 2));
            $revision      = bcsub($bestValue, (bcmul($dividedValue, $qtdItens, 2)), 2);
            $additionLeft = $revision;

            foreach ($divisionByItems as $item) {
                if($cart['productsPrint'] == 'N') {
                    if(empty($item['childIndex'])) {
                        $cart['selectedProducts'][$item['index']]['addition'] = $dividedValue;
                        $price = $cart['selectedProducts'][$item['index']]['price'];

                        if($additionLeft > 0) {
                          $price = floatval(bcadd($price, $additionLeft, 2));
                          $addition = floatval(bcadd($dividedValue, $additionLeft, 2));
                          $cart['selectedProducts'][$item['index']]['addition'] = $addition;
                          $additionLeft = 0;
                        }

                        $cart['selectedProducts'][$item['index']]['price'] = floatval(bcadd($price, $dividedValue, 2));

                        
                    } else {
                        $cart['selectedProducts'][$item['index']]['childItems'][$item['childIndex']]['addition'] = $dividedValue;
                        $price = $cart['selectedProducts'][$item['index']]['childItems'][$item['childIndex']]['price'];

                        if($additionLeft > 0) {
                          $price = floatval(bcadd($price, $additionLeft, 2));
                          $addition = floatval(bcadd($dividedValue, $additionLeft, 2));
                          $cart['selectedProducts'][$item['index']]['childItems'][$item['childIndex']]['addition'] = $addition;
                          $additionLeft = 0;
                        }

                        $cart['selectedProducts'][$item['index']]['childItems'][$item['childIndex']]['price'] = floatval(bcadd($price, $dividedValue, 2));
                    }
                } else {
                    $productsValue = floatval(bcadd($productsValue, $cart['fullPrice'], 2));
                    $totalAddition = $bestValue;
                    $additionLeft  = $totalAddition;

                    foreach ($divisionByItems as $item) {
                        $vrAdd = bcmul((bcdiv($item['price'], $productsValue, 6)), $totalAddition, 2);
                        $price  = $cart['selectedProducts'][$item['index']]['price'];

                        $cart['selectedProducts'][$item['index']]['price'] = floatval(bcadd($item['price'], $vrAdd, 2));
                        $cart['selectedProducts'][$item['index']]['addition'] = floatval($vrAdd);

                        $additionLeft = bcsub($additionLeft, $vrAdd, 2);
                    }
                }
            }
        }

        if($type == 'D' && $qtdItens) {
            if($cart['productsPrint'] == 'N') {
                $dividedValue  = floatval(bcdiv($bestValue, $qtdItens, 2));
                $totalDiscount = $bestValue;
                $discountLeft  = $totalDiscount;

                if($bestValue > $productsValue) {
                    throw new \Exception('Valor calculado de desconto é superior ao valor total de produtos.');
                }

                foreach($divisionByItems as &$prod) {
                    if($prod['price'] > $dividedValue) {
                        $prod['price']    = bcsub($prod['price'], $dividedValue, 2);
                        $prod['discount'] = bcadd($prod['discount'], $dividedValue, 2);
                        $discountLeft     = bcsub($discountLeft, $dividedValue, 2);
                    } else {
                        $maxDiscount      =  bcsub($prod['price'], '0.01', 2);
                        $prod['price']    = bcsub($prod['price'], $maxDiscount, 2);
                        $prod['discount'] = bcadd($prod['discount'], $maxDiscount, 2);
                        $discountLeft     = bcsub($discountLeft, $maxDiscount, 2);
                    }
                }

                if(isset($prod))
                    unset($prod);

                if($discountLeft > 0) {
                    foreach($divisionByItems as &$prod) {
                        if($prod['price'] > $discountLeft) {
                            $prod['price']    = bcsub($prod['price'], $discountLeft, 2);
                            $prod['discount'] = bcadd($prod['discount'], $discountLeft, 2);
                            $discountLeft     = 0;
                        } else {
                            $maxDiscount      =  bcsub($prod['price'], '0.01', 2);
                            $prod['price']    = bcsub($prod['price'], $maxDiscount, 2);
                            $prod['discount'] = bcadd($prod['discount'], $maxDiscount, 2);
                            $discountLeft     = bcsub($discountLeft, $maxDiscount, 2);
                        }

                        if($discountLeft == 0) {
                            break;
                        }
                    }

                    if(isset($prod))
                        unset($prod);

                    if ($discountLeft > 0) {
                        throw new \Exception('Erro de precisao na distribuição de descontos.');
                    }
                }
                
                foreach ($divisionByItems as $item) {
                    if(empty($item['childIndex'])) {
                        $price = $cart['selectedProducts'][$item['index']]['price'];

                        $cart['selectedProducts'][$item['index']]['price'] = floatval($item['price']);
                        $cart['selectedProducts'][$item['index']]['discount'] = floatval($item['discount']);
                    } else {
                        $price = $cart['selectedProducts'][$item['index']]['childItems'][$item['childIndex']]['price'];

                        $cart['selectedProducts'][$item['index']]['childItems'][$item['childIndex']]['price'] = floatval($item['price']);
                        $cart['selectedProducts'][$item['index']]['childItems'][$item['childIndex']]['discount'] = floatval($item['discount']);
                        
                    }
                }
            } else {
                $productsValue = floatval(bcadd($productsValue, $cart['fullPrice'], 2));
                $totalDiscount = $bestValue;
                $discountLeft  = $totalDiscount;

                foreach ($divisionByItems as $item) {
                    $vrDesc = bcmul((bcdiv($item['price'], $productsValue, 6)), $totalDiscount, 2);
                    $price  = $cart['selectedProducts'][$item['index']]['price'];

                    $cart['selectedProducts'][$item['index']]['price'] = floatval(bcsub($item['price'], $vrDesc, 2));
                    $cart['selectedProducts'][$item['index']]['discount'] = floatval($vrDesc);

                    $discountLeft = bcsub($discountLeft, $vrDesc, 2);
                }
                
                $fatherDiscount = floatval(bcsub($fatherDiscount, $discountLeft, 2));
                $fatherDiscount = floatval($discountLeft);
            }
        }
        

        $cart['discount'] = floatval(bcadd($cart['discount'], $fatherDiscount, 2));
        $cart['addition'] = floatval(bcadd($cart['addition'], $fatherAddition, 2));

        $cart['price'] = floatval(bcsub((bcadd($cart['price'], $fatherAddition, 2)), $fatherDiscount, 2));

        if($cart['productsPrint'] == 'S' && $cart['price'] < 0.01) {
            $cart['price'] = 0.01;
        }

        $response->addDataSet(
            new DataSet('', $cart)
        );
    } catch (\Exception $e) {
        $message = 'Erro no calculo de campanhas promocionais. ' . $e->getMessage();
        $response->addMessage(new Message($message, self::ERROR_MESSAGE));
    }
  }
}